C     -- usage e.g.: plates 1.8 2.1 2 1414771 10 0.4 0.6 rad=1.732
C     -- usage e.g.: plates 1.8 2.1 2 1414771 10 

      program plates
C     ==== Looks for LSS in individual `Schmidt plates' from QSO data.
C     A 1-D fourier transform is used to find evidence of LSS in the
C     invididual `fields'. ====
      
      integer*4 mxqso,nqso
c      parameter (mxqso=11358)	! Number of records
      include 'mxqso.dec'
c      real*8        ra_        (mxqso) ! (deg) Right Ascension J2000
c      real*8        dec        (mxqso) ! (deg)     Declination J2000

      real*8        ra_d,decd
      real*8        div1,div2   ! divisions in ra or dec


c      character*17  name       (mxqso) ! Most common name of the object
c      character*1   l_z        (mxqso) ! [&gt;*] limit or method flag on z (4)
c      real*4        redsh          (mxqso) ! ? redshift
c      real*4        Vmag(mxqso),B_V   (mxqso) ! V, B-V 

*Note (4): n_z may indicate a lower limit on z, or
*     '*', if the redshift is measured by slitless spectroscopy.

C     ---- curvature parameters, pair distances ----
c      parameter(mxcurv=300) !! WARNING: present in findAB !!
      include 'mxcurve.dec'

c     --- !!!! WARNING: cc_0 is really w_quintessence if do_quin=.true. !!!! ---
      real*4   om_0(mxcurv),cc_0(mxcurv),H_0  
c      real*4   pdWmax(mxcurv,mxcurv)

      integer*4 nbin(mxcurv,mxcurv)

C     ---- differences between adjacent bins ---
C     3rd index: 1=mean 2=mean abs 3=sig 4=max
C     !!!    4th index: 1=obs  2=boot mean  3=poisson mean

c      parameter(nctrl=10)
c      parameter(mxboot=1+nctrl*2)
c      parameter(mxboot=1+1)
      real*4        A(mxboot),Asig(mxboot),B(mxboot),Bsig(mxboot)
      real*4        chi2n(mxboot)
      real*4        Am,Amsig,Bm,Bmsig

      real*4   diffstat(mxcurv,mxcurv,5,mxboot) 


C     INTERNAL:
      real*4        ra_4(mxqso),dec4(mxqso),gall(mxqso),galb(mxqso)
      real*4        pdW(mxqso)
      parameter(ntwo=2,mxmran=20,mxbigq=mxmran*mxqso)
c      real*4        red_scram(mxqso*mxmran,ntwo)  ! 1=orig, 2=scrambled
c      real*8        w_1(mxqso*mxmran,ntwo),x_1(mxqso*mxmran,ntwo)
c      real*8        y_1(mxqso*mxmran,ntwo),z_1(mxqso*mxmran,ntwo)
      real*4        red_scram(mxbigq,ntwo)  ! 1=orig, 2=scrambled
      real*8        w_1(mxbigq,ntwo),x_1(mxbigq,ntwo)
      real*8        y_1(mxbigq,ntwo),z_1(mxbigq,ntwo)

c     ---- following is to distinguish radial from tangential pairs ----
c     -- added 22/06/00 --
c      parameter(mxrt=2)  ! 1= radial 2=tangential
c      real*8        w_1rt(mxrt,mxbigq,ntwo),x_1rt(mxrt,mxbigq,ntwo)
c      real*8        y_1rt(mxrt,mxbigq,ntwo),z_1rt(mxrt,mxbigq,ntwo)

      real*8        pdW_1(mxbigq,ntwo), pdi,pdj,pdmean
      real*8        rcurv8
      real*8        decdi,decdj,ra_di,ra_dj, d_ra_,d_dec
      real*8        dthe_d
      real*8        dprad,dptan
      real*8        tol_dptan
      parameter(tol_dptan=1d-4)
      real*8        rtrat8

      real*4        ra_scram(mxbigq),decscram(mxbigq)


      real*8        chid,dd

c     ---- following added 18/07/00 - required for linux g77 ---
      real*8        angle4D8
      external      angle4D8
      real*8        coshd,sinhd,sinhinvd
      external      coshd,sinhd,sinhinvd

c      parameter(mxpair=(mxqso*(mxqso-1))/2)
c      logical*1  isclose(mxqso,mxqso) ! is the pair "close" for some om,lam?

      integer*4     irefs(mxqso) ! for selection by published references

      parameter(mxflds=15)
      real*4        fields(4,mxflds)
      real*4        flds_Iov(4,2) ! the two values used in RM00 a-ph/9911413
      real*4        flds_2dF(4,9) ! options for 2dF 0003206
      real*4        flds_allsky(4,1) ! options for 2dF 0003206
c     ---- for the 1d fourier analysis ----
      parameter(mxbins=2048 *8)
      real*4        r(mxbins),rfreq(-mxbins/2:mxbins/2)
      real*4        dN(mxbins)
      real*4        dNfour(-mxbins/2:mxbins/2)

      complex       dNc(mxbins)

c     ---- for the 2pt spatial corrn function ----
      parameter(nthree=3)
      real*4        histog(mxbins,nthree)  ! 1=DD, 2=DR, 3=RR

      real*8        hist8(mxbins,nthree) ! 1=DD, 2=DR, 3=RR
      real*8        fmore1_8,fmore2_8
      real*8        paircount(nthree)

      real*4        corrn(mxbins), corrn_cp(mxbins)
      parameter(mxb_sm=1050)! 150) !300) !30)
      real*4        corrnall(mxcurv,mxcurv,mxb_sm,mxboot)

c     ! 1= real;  2=mean bootstrap  3=sigma bootstrap
c     !           4=mean poisson    5=sigma poisson
C     !  6, 7 for plotting
      real*4        dNf_all(-mxbins/2:mxbins/2,7)

      character*1   ch
      character*3   ch3
      character*5   ch5a,ch5b,ch5c
      character*13  chfour
      character*256  string

      logical*4     test, test0
      logical*4     transp ! used for findAB 
      integer*4     idirn ! 1= radial, 2= r.a., 3=dec
      logical*4     dogal ! do galactic coordinate cut ?

c      logical*4     ranboot ! superceded: randomise before bootstrapping Y/N ?

      logical*4     ranboot ! randomise (uniformly) =Y or boot =N


      logical*4     radivide ! divide ra into fields before randomising
      logical*4     dedivide    ! divide dec into fields before randomising
      logical*4     zscramble   ! test for GAM
      logical*4     uniquez     ! to see if effect just in angle
      logical*4     plotdN      ! plots dN for _every_ om_0, cc_0 value

      logical*4     do_cutalpha ! make "alpha" set of Iov distinct from "dec"?
      logical*4     only1radiv  ! only 1 "alpha" subdivision of Iov ?
      logical*4     uniform2d   ! rand uniform distbn for 2nd dimension ?
      logical*4     samez       ! make "rand" z identical to originals ? 
      logical*4     avoidRRcross ! avoid cross terms between RR sets (e.g. 2dF)

      logical*4     do_radtan  ! separate radials from tans ?
      logical*4     radvstan    ! rad = .true. , tan = .false.
      real*4        radtanratio   ! minimum ratio e.g. sqrt(3) => 30
      logical*4     pythagOK   ! pythag approx OK for rad/tan separation?
      logical*4     radtantest ! for local use at point of test

      logical*4     doIov_raw   ! do Iovino "raw" data set ?
      logical*4     do2dF       ! do 2dF astro-ph/0003206 ?
      logical*4     doRLAGN     ! do RLAGN list ?
      logical*4     doeqzs      ! do nearly equal z's only (for topo)?

      logical*4     docorrn     ! calculate a spatial 2pt correlation fn ?
      logical*4     dosubflds   ! do sub fields of 2dF ?
      logical*4     dosmoodNdz  ! do smooth dNdz instead of scramble ?
      logical*4     domoreran   ! have more "Randoms" than "Data" points ?

      logical*4     do_quin     ! do quintessence ?

      logical*4     modif_y     ! to modify histograms ! default = .false.

      real*4        dz_equal    ! delta z if "equal z's" only are used

c      parameter(c=299792.4562)
c      parameter(conh=3000.0,tolcurv=0.01,tolerdd=0.2)

      parameter(conh=2997.924562) ! added 23/06/00
      parameter(tolcurv=0.01,tolerdd=0.2)
      parameter(pi=3.14159265359,pi_180=pi/180.0)
      parameter(pid=3.14159265359,pid_180=pi/180.0)


c      common /Vercomm/ nqso, idum,ra_,dec,redsh,Vmag,B_V,l_z,name

      data test0 / .false./ !.true./ !
      data dogal / .false./ !.true./ !
c      data ranboot / .false./ !.true./ !
      data radivide / .true./ !.false./!
      data dedivide /.true./ ! .false./  !
      data zscramble / .false./!/.true./ !!name misleading! just a test !
      data uniquez /.false./ !.true./ ! 
      data plotdN / .false./ !.true./ !

      data docorrn/.true./ ! .false./    !

      data do_cutalpha/ .false./    !.true./ !
      data only1radiv/.true./  ! .false./    !

c     --- a double check that the following two options work is
c     to set  uniform2d = .false. and samez = .true. - this should
c     give a correlation function exactly identical to zero ---

c     -- radial:     uniform2d=.false., samez=.false. 
c     -- tangential: uniform2d=.true., samez=.true. 
      data uniform2d/.false./    ! .true./ !
      data samez/.false./    !.true./ !  only done for icheck=1 so far !! 
      data pythagOK /.true./ ! hardwired, alternative slow !

      data avoidRRcross /.false./! .true./  !  

      data doIov_raw / .true./  !.false./    !
      data do2dF/.false./       !.true./ ! 
      data doRLAGN/.true./      ! .false./    !
      data doeqzs/.true./      ! .false./    !
      data dz_equal/0.01/ 

      data dosubflds/.false./   ! .true./ !
      data dosmoodNdz/.false./  !.true./ ! 
      data domoreran/.true./ ! .false./    !

      data do_quin/.true./ ! .false./    !
      
      data modif_y /.false./  ! hardwired, do not modify modif_y !
      data zsmmin,zsm/0.05, 0.15/

      data iwarn1,iwarn2/ 2,2/ ! warnings on close or distant pairs


      common /testcomm/ test

      data nfields/1/ !15/
C     -- thetamax in radians --
      data thetamax/0.24/ ! ra sample
c      data thetamax/0.24/ ! dec sample
c      data thetamax/6.2831853/ !0.2/ !1.2/ !0.4/ !4.184 /!0.17/!0.24/ !6.07/!
      data fields/ 
c     _     -0.1,360.1,-30.0,22.0, ! galactic coords
c     _     -0.1,360.1,-20.0,-10.0, ! galactic coords
c     _     -0.1,360.1,10.0,20.0, ! galactic coords
c     _     -0.1,240.1, 30.0,90.1, ! galactic coords for 'PG' objects
c     _     -0.1,360.1,-45.0,45.0, ! galactic coords
c     _     -0.1,360.1,-90.1,90.1,  ! 4\pi = all sky 
c     _     130.0,230.0,-40.0,30.0, ! eq + 'PKS' subcat
c     _     155.0,260.0,46.2,47.8,
c     _     10.0,30.0,-43.0,-37.0, ! 1 +2+3
c     _     10.0,30.0,-42.5,-37.0, ! 1 +2+3  ! using  -42.5 not -43.0
c     _     10.5,29.2,-42.0,-37.5, ! 1 +2+3  ! IovinoCS96 \alpha sample ! old
c     _     10.5,29.75,-42.0,-37.5, ! 1 +2+3  ! IovinoCS96 \alpha sample ! new
     _     10.5,15.0, -42.0,-28.0, ! 2+4 +6! IovinoCS96 \delta sample
c     _     10.5,15.0, -42.0,-32.3, ! 2+4 +6! IovinoCS96 \delta 2 plates only !
c     _     172.0,190.0,-11.5,-11.0, !7  ! the -11 slice !
c     _     23.0,30.0,-43.0,-37.0, ! 1 
     _     56*-99.9/
C     _     10.0,16.0,-43.0,-37.0, ! 2
C     _     16.0,23.0,-43.0,-37.0, ! 3
C     _     10.0,16.0,-37.0,-33.0, ! 4
C     _     16.0,23.0,-37.0,-33.0, ! 5
C     _     10.0,16.0,-31.0,-26.0, ! 6
C     _     172.0,190.0,-11.5,-11.0, !7
C     _     0.0,8.0,-3.0,3.0,
C     _     13.0,22.0,-3.0,3.0,
C     _     41.0,46.0,-3.0,3.0, ! 10
C     _     203.5,205.0,26.0,28.5,
C     _     249.0,251.5,39.0,41.0, !12
C     _     320.0,326.0,-47.0,-42.0,
C     _     329.0,335.0,-22.0,-16.0/ !14
c     _     -0.1,360.1,-90.1,90.1,
c     _     24*-99.9/

      data flds_Iov/
     _     10.5,29.75,-42.0,-37.5, ! 1 +2+3  ! IovinoCS96 \alpha sample ! new
     _     10.5,15.0, -42.0,-28.0/ ! 2+4 +6! IovinoCS96 \delta sample


      data flds_2dF/
c     ---- foll are for "x-y theta's -> r.a." ----
c     _     270.0,280.0, -31.0,-29.0, ! SGP - small subset for testing
c     _     228.0,312.0, -31.0,-29.0, ! SGP, assumed at dec=-30 exactly 
c     _     53.0, 128.0, -1.0,1.0/ ! NGP, assumed at dec=0 exactly 
c     _     228.0,312.0, -33.0,-27.0, ! SGP,  at dec=-30 approx
c     _     53.0, 128.0, -3.0,3.0/ ! NGP, at dec=0 approx
c     ---- foll. are based on p1 of astro-ph/0003206 ----
c     _     -33.0, 55.0, -33.0, -27.0, ! SGP 
c     _     147.0, 225.0, -3.0, 3.0,  ! NGP
c     _     147.0, 172.0, -3.0, 3.0, ! NGPa = old3 = new 1
c     _     172.0, 192.0, -3.0, 3.0,  ! NGPb = old4  = new 2
c     _     192.0, 210.0, -3.0, 3.0,  ! NGPc = old5  = new 3
c     _     210.0, 225.0, -3.0, 3.0,  ! NGPd = old6 = new 4
c     _     -33.0, -23.0, -33.0, -27.0, ! SGPa = old7  = new 5
c     _     -23.0, 0.0, -33.0, -27.0, ! SGPb = old8  = new 6
c     _     0.0, 35.0, -33.0, -27.0, ! SGPc = old9  = new 7
c     _     35.0, 55.0, -33.0, -27.0, ! SGPd = old10  = new 8
c     _     -33.0, 225.0, -33.0, 3.0/  ! both complete fields
c     ---- foll. are for the Smith 000628 version ----
c     _     147.5, 172.0, -3.0,3.0, ! NGP 1
c     _     172.0, 197.0, -3.0,3.0, ! NGP 2
c     _     197.0, 222.0, -3.0,3.0, ! NGP 3
c     _     -35.0, -7.0, -33.0, -27.0, ! SGP 4
c     _     -7.0,  21.0, -33.0, -27.0, ! SGP 5
c     _     21.0,  49.0, -33.0, -27.0, ! SGP 6
c     _     -35.0, 49.0, -33.0, -27.0, ! SGP all = 7
c     _     147.5, 225.0, -3.0, 3.0,  ! NGP all = 8
c
c     --- these are for the grotesquely incomplete (by coverage) 
c     "10K" catalogue Croom et al. astro-ph/0104095 ---
     _     147.0, 155.0, -3.0, 3.0, ! NGP = new 1
     _     172.0, 171.0, -3.0, 3.0,  ! NGP = new 2 null
     _     192.0, 191.0, -3.0, 3.0,  ! NGP  = new 3 null
     _     203.0, 216.0, -3.0, 1.0,  ! NGP = new 4
     _     -35.0, -30.0, -33.0, -28.0, ! SGP = new 5
     _     -17.0, -2.5, -33.0, -27.0, ! SGP = new 6
     _     21.0, 31.0, -33.0, -27.0, ! SGP  = new 7
     _     40.0, 50.0, -33.0, -27.0, ! SGP = new 8

     _     -35.0, 225.0, -33.0, 3.0/  ! both complete fields


      data flds_allsky/
     _     -0.01,360.01,-90.01,90.01/ ! all sky 


c      call cutVeron(320.0,326.0,-47.0,-42.0)

      test=test0! .true. !

c      print*,'test: ra_(5)=',ra_(5)
c      print*,'test B: ra_(5)=',ra_(5)
c
c      print*,'nqso=',nqso

      zmax=5.0

C     --- idirn ! 1= radial, 2= r.a., 3=dec  ---
c      idirn=3 ! 2                   !1 ! 

      om1=0.1 !0.4 !0.3 ! 0.3 !
      om2=0.5 !   1.1! 2.0 !1.0 !0.55 !0.55 !
      cc1=0.5 !-1.0 !0.0 !0.45 !
      cc2=0.9 !1.0 !1.0 !0.6 !0.7 !0.7 !

      if(do_quin)then
         ww1=-1.1
         ww2=0.0
      endif


c      om1= 0.3 !0.2! 1.0 !0.26!
c      om2=1.0 !0.55
c      cc1= 0.7 !0.45! 0.8 !0.0!
c      cc2=0.0! 0.7

c      om1=0.1
c      om2=0.5
c      cc1=0.5 !0.65 !0.0
c      cc2=0.9
      
c     ---- essayer proche du point optimum ?? ----
c      om1=0.2
c      om2=0.4
c      cc1=0.6
c      cc2=0.8

c      z1= 1.95!1.8 !2.1 !0.5 !2.4 !
c      z2= 2.10!2.4 !1.8 !5.0 !3.0 !2.1 !3.2 !

C     ---- Warning: following only works on systems supporting getarg ----
      call getarg(1,string) ! z1
      read(string,*)z1
      call getarg(2,string) ! z2
      read(string,*)z2
      print*,'z1,z2=',z1,z2

c     --- idirn ! 1= radial, 2= r.a., 3=dec  ---
c     --- i2dF ! 1=SGP 2=NGP 3=both ---
C       --- free format * required for linux double-pentium oberon@camk.

      call getarg(3,string) ! ii (which dirn or sub-catalogue)
c      read(string,'(i)')ii
      read(string,*,err=331)ii
 331  continue
      print*,'ii=',ii

C     ==== DEFAULTS ====

      nflush=1312656            !1112656 ! 1427656 ! 342125
c      om1=0.3
c      cc1=0.7
      moreran=1  ! ratio of number of randoms to data points

      call getarg(4,string) ! nflush
c      read(string,'(i)',err=332)nflush
      read(string,*,err=332)nflush
 332  continue
      call getarg(5,string) ! moreran
c      read(string,'(i)',err=333)moreran
      read(string,*,err=333)moreran
 333  continue
      call getarg(6,string) ! om1 
      read(string,*,err=334)om1
 334  continue
      call getarg(7,string) ! cc1
      read(string,*,err=335)cc1
 335  continue

      print*,'ii,nflush,moreran,om1,cc1=',ii,nflush,moreran,om1,cc1
      print*,' '
      if(do_quin)then
         ww1=cc1
         print*,'ww1=',ww1
      endif

      call getarg(8,string) ! "rad" or "tan" 
      ch3=string(1:3)
      if(ch3.eq.'rad')then
         do_radtan=.true.
         radvstan=.true.
         read(string(5:256),*)radtanratio
         rtratio=radtanratio
         rtrat8=radtanratio
         print*,'rtratio=',rtratio
      elseif(ch3.eq.'tan')then
         do_radtan=.true.
         radvstan=.false.
         read(string(5:256),*)radtanratio
         if(radtanratio.gt.tol_dptan)then
            rtratio=1.0/radtanratio
            rtrat8=1.0d0/radtanratio
         else
            stop 'radtanratio too small'
         endif
      else
         do_radtan=.false.
         print*,'No recognisable argument 8 read from command line.'
      endif
      print*,'do_radtan,radvstan=',do_radtan,radvstan

      print*,'nflush,moreran,om1,cc1=',nflush,moreran,om1,cc1

      if(.not.(do2dF))then
         idirn=ii
      elseif(do2dF)then
         i2dF=ii
      endif

      if(do2dF)then
         idirn=2
         radivide=.false.
         docorrn=.true.
         thetamax= 2.0

c         nflush=1312656 !1112656 ! 1427656 ! 342125

      endif

      if(doRLAGN)then           ! added 24/09/2001
         idirn=2 ! is this necessary???
         radivide=.false.
         docorrn=.true.
         thetamax=3.1416
      endif

c     ---- moved out of (if(do2dF)) 18/06/00 ----
         do i=1,nflush
            xxx=randt()
         enddo
      

      if(idirn.eq.2.or.idirn.eq.3)then
         if(.not.(do2dF))then
            if(do_cutalpha)then
               flds_Iov(1,1)= flds_Iov(2,2)
            endif

            do i=1,4
               fields(i,1)=flds_Iov(i,idirn-1)
            enddo
         elseif(idirn.eq.2)then
            do i=1,4
               fields(i,1)=flds_2dF(i,i2dF)
            enddo
         endif
      else
         print*,'WARNING: idirn=',idirn,' is not a standard option.'
      endif

      if(doRLAGN)then           ! added 24/09/2001
         do i=1,4
            fields(i,1)=flds_allsky(i,1)
         enddo
      endif


      Rlss0= 130.0 !/2.0
      nom=11!10!30 !20 !  1 ! 5!
      ncc=11!   10!30 !20 ! 1 !5!
      H_0=100.0 
      
      if(do_quin)then
         nww= ncc
      endif


      if(idirn.eq.1)then
         nbins=128              !mxbins
      else
         nbins=1024 !2048 !128 !
      endif

      do jj=1,5
         do i=-nbins/2,nbins/2
            dNf_all(i,jj)=0.0
         enddo
      enddo
      
      
      if(nom.gt.1)then
         dom=real(om2-om1)/real(nom-1)
      else
         dom=-99.9
      endif
      if(ncc.gt.1)then
         dcc=real(cc2-cc1)/real(ncc-1)
      else
         dcc=-99.9
      endif
      print*,'dom,dcc=',dom,dcc

      if(do_quin)then
         if(nww.gt.1)then
            dww=real(ww2-ww1)/real(nww-1)
         else
            dww=-99.9
         endif
      endif


      do ifi=1,1 !1,nfields !7,7 !
         print*,' '


         do ib=1,mxboot
            do istat=1,5
               do iom=1,nom
                  do icc=1,ncc
                     diffstat(icc,iom,istat,ib)=0.0
                  enddo
               enddo
            enddo
         enddo

         do icheck=1,mxboot
            if(icheck.eq.1)then
               ictrl=1
            elseif(2.le.icheck.and.icheck.le.1+nctrl)then
               ictrl=2
               ranboot=.false.
            else
               ictrl=3
               ranboot=.true.
            endif
               

            if(test)print*,'will call readVeron'
            if(ifi.eq.1.and.icheck.eq.1)then
               if(do2dF)then
                  print*,'will call read2dF_0003206...'
c                  call read2dF_0003206
                  call read2dF_0104_10K
               elseif(doRLAGN)then
                  print*,'will call rdRLAGN...'
                  call rdRLAGN
               elseif(.not.(doIov_raw))then
                  call readVeron
               else
                  call readIovino
                  call smooIov ! added 16/06/00, hardwired
               endif
               if(.not.(doIov_raw.or.do2dF))then
                  call cleanVeron
               elseif(do2dF)then
                  print*,'will call clean2dF...'
                  call clean2dF

                  print*,'will call complete2dF...'
                  call complete2dF(0.8)
               endif

               call bkupVeron
            else
               call rdmemVeron
            endif


            string='PG'
            len=2
            string='PKS'
            len=3
c            call subcatVeron(len,string)

c            if(icheck.ge.2) call ranzVeron
            if(icheck.ge.2)then
               if(idirn.eq.1)then
                  call GauzVeron !! WARNING should be better after cut
               endif
            endif

            if(test)print*,'Will do cutgVeron or cutveron'

            if(dogal)then       ! galactic coords cut
               call cutgVeron(fields(1,ifi),fields(2,ifi),
     _              fields(3,ifi),fields(4,ifi),
     _              gall,galb)
            
            else                ! equatorial coords cut
               call cutVeron(fields(1,ifi),fields(2,ifi),
     _              fields(3,ifi),fields(4,ifi))
            endif
c            call cutVeron(fields(1,ifi),fields(2,ifi),
c     _           fields(3,ifi),fields(4,ifi))


            
            print*,'will call cutzVeron(z1,z2): ',z1,z2
            call cutzVeron(z1,z2)

            dra = fields(2,ifi)-fields(1,ifi)
            ddd = fields(4,ifi)-fields(3,ifi)
            ddmean = (fields(4,ifi)+fields(3,ifi))/2.0
            print*,'ra interval is ',dra, 'coord degrees'
            print*,'ra interval is ',
     _           dra*cos(ddmean*pi_180), 'gr circle degrees'
            print*,'solid angular number density is',
     _           real(nqso)/(dra*cos(ddmean*pi_180)*ddd)
            write(6,'(a,f5.1,a,f5.1,a,f5.1,a)')
     _           '\\LaTeX: \\& ', dra,
     _           ' \\& ',  dra*cos(ddmean*pi_180),
     _           ' \\& ', real(nqso)/(dra*cos(ddmean*pi_180)*ddd)
            print*,' '

            irefs(1)=729 ! Iovino et al 1996
            irefs(2)=433 ! Drinkwater 1987
            nref=2
            
            if(.not.(doIov_raw.or.do2dF))then
               call cutrefVeron(1,irefs(1))
            endif

            if(icheck.eq.1.and.dosubflds)then
               iq=0
               do jf=1,nfldtot
                  ipf=0
                  do while(iq+ipf.lt.nqso.and.ifld(iq+ipf+1).eq.jf)
                     ipf=ipf+1
                  enddo
c     test only 
c                  print*,'After jf search: jf=',jf
c                  print*,'iq,ipf,ifld(iq+ipf),ifld(iq+ipf+1)='
c                  print*,iq,ipf,ifld(iq+ipf),ifld(iq+ipf+1)

c                  if(iq+ipf.lt.nqso)ipf=ipf-1
                  if(ipf.ne.npfld(jf))then
                     print*,'minor warning: ipf < npfld(jf), jf=',
     _                    ipf, npfld(jf), jf
                     npfld(jf)=ipf
                  endif
                  iq=iq+ipf
               enddo

               jj=0
               do jf=1,nfldtot
                  jj=jj+npfld(jf)
               enddo
               print*,'Ended revision of npfld. jj=',jj
            endif


            if(1+1.eq.1)then
               do iq=1,nqso
                  write(6,'(a,a17,3f8.1)')
     _                 "AA ", name(iq),ra_(iq),dec(iq),redsh(iq)
               enddo
               stop
            endif


C     --- !! test only !! ---
            if(uniquez)then
               do iq=1,nqso
                  redsh(iq)= (z1+z2)/2.0
               enddo
            endif

            if(icheck.ge.2)then
               if(idirn.eq.2)then
                  if(dogal)then
                     if(ranboot)then
                        print*,'will call ranx4Veron: ',
     _                       fields(1,ifi),fields(2,ifi)
                        call ranx4Veron(nqso,gall,
     _                       fields(1,ifi),fields(2,ifi))
                     endif

c                        call bootVeron  
c     ---  foll. is new version 19/06/00 - avoids extra noise in randoms
                        if(.not.(ranboot))call bootVeron  
c     --- 

                  else
                     if(zscramble)then           !! test  only !!
                        call ranzVeron
                     else
                        if(ranboot)then
                           if(do2dF)then
                              do iq=1,nqso
                                 red_scram(iq,1)=redsh(iq)
                              enddo
c     ---- warning: modifies data in  redsh( ) ----
                              if(.not.(dosubflds))then
                                 call scramble
     _                                (nqso,red_scram(1,1),redsh)
                              else
                                 iq=0
                                 do jf=1,nfldtot
                                    if(.not.(dosmoodNdz))then
                                       call scramble(
     _                                      npfld(jf),
     _                                      red_scram(iq+1,1),
     _                                      redsh(iq+1))
                                    else
                                       call smoothdNdz(
     _                                      npfld(jf), zsmmin,zsm,
     _                                      red_scram(iq+1,1),
     _                                      redsh(iq+1))
                                    endif
                                    iq=iq+npfld(jf)
                                 enddo
                                 if(iq.ne.nqso)then
                                    print*,'plates: iq, nqso=',iq,nqso,
     _                                   ' => WARNING !'
                                 endif
                              endif !  if(.not.(dosubflds))then

                           elseif(.not.(radivide))then
                              print*,'will call ranx8Veron: ',
     _                             fields(1,ifi),fields(2,ifi)

                              call ranx8Veron(nqso,ra_,
     _                             dble(fields(1,ifi)),
     _                             dble(fields(2,ifi)) )

                              if(uniform2d)then ! added 06/06/00
                                 call ranx8Veron(nqso,dec,
     _                                dble(fields(3,ifi)),
     _                                dble(fields(4,ifi)) )
                              endif
                           elseif(radivide)then
c                              div1= 16.25d0 ! 17.0d0
c                              div2= 22.75d0 ! 23.0d0

                              div1= 16.75d0
                              div2= 23.25d0

                              i1=icount8Veron(nqso,ra_, 
     _                             dble(fields(1,ifi)),div1)
                              i2=icount8Veron(nqso,ra_, div1,div2)

                              i3=icount8Veron(nqso,ra_, div2,
     _                             dble(fields(2,ifi)))
                              
                              call ranx8Veron(i1,ra_(1),
     _                             dble(fields(1,ifi)),div1)
                              call ranx8Veron(i2,ra_(i1+1),div1,div2)

                              call ranx8Veron(i3,ra_(i1+i2+1),div2,
     _                             dble(fields(2,ifi)))

                              if(uniform2d)then ! added 06/06/00
                                 call ranx8Veron(nqso,dec,
     _                                dble(fields(3,ifi)),
     _                                dble(fields(4,ifi)) )
                              endif

                           endif !if(do2dF)then

                           call scramble(nqso,redsh,redsh) ! i.e. ran uniform

                        endif   !if(ranboot)then

                        if(uniquez)then
                           do iq=1,nqso
                              redsh(iq)= (z1+z2)/2.0
                           enddo
                        endif

c                        call bootVeron  
c     ---  foll. is new version 19/06/00 - avoids extra noise in randoms
                        if(.not.(ranboot))call bootVeron  
c     --- 

                     endif      !if(dogal)then
                  endif         !               if(idirn.eq.2)then
                  

               elseif(idirn.eq.3)then
                  if(dogal)then
                     stop 'not yet implemented'
                     if(ranboot)then
                        print*,'will call ranx4Veron: ',
     _                       fields(1,ifi),fields(2,ifi)
                        call ranx4Veron(nqso,gall,
     _                       fields(1,ifi),fields(2,ifi))
                     endif

c                        call bootVeron  
c     ---  foll. is new version 19/06/00 - avoids extra noise in randoms
                        if(.not.(ranboot))call bootVeron  

c     --- 

                  else
                     if(zscramble)then !! test  only !!
                        call ranzVeron
                     else
                        if(ranboot)then
                           if(.not.(dedivide))then
                              print*,'will call ranx8Veron: ',
     _                             fields(3,ifi),fields(4,ifi)

                              call ranx8Veron(nqso,dec,
     _                             dble(fields(3,ifi)),
     _                             dble(fields(4,ifi)) )

                              if(uniform2d)then ! added 06/06/00
                                 call ranx8Veron(nqso,ra_,
     _                                dble(fields(1,ifi)),
     _                                dble(fields(2,ifi)) )
                              endif

                           elseif(dedivide)then
                              div1= -37.5d0
                              div2= -32.5d0
                              i1=icount8Veron(nqso,dec,
     _                             dble(fields(3,ifi)), div1)
                              i2=icount8Veron(nqso,dec, 
     _                             div1,div2)
                              i3=icount8Veron(nqso,dec, 
     _                             div2,dble(fields(4,ifi)))
                              
                              call ranx8Veron(i1,dec(1),
     _                             dble(fields(3,ifi)), div1)
                              call ranx8Veron(i2,dec(i1+1),
     _                             div1,div2)
c                              call ranx8Veron(i3,dec(i1+1),
                              call ranx8Veron(i3,dec(i1+i2+1),!corrected 22/11/99
     _                             div2,dble(fields(4,ifi)))


c     --- !! test only !! test only !! NONPHYSICAL NONPHYSICAL !!--- 
c                                 call ranx8Veron(nqso,dec,
c     _                                dble(fields(4,ifi))-1.0,
c     _                                dble(fields(4,ifi)) )
c     --- !! test only !! test only !! --- 


                              if(uniform2d)then ! added 06/06/00
                                 call ranx8Veron(nqso,ra_,
     _                                dble(fields(1,ifi)),
c     _                                dble(fields(2,ifi)) -1.0,!! test only !!
     _                                dble(fields(2,ifi)) )
                              endif

                           endif

c     --- foll added 19/06/00
                           call scramble(nqso,redsh,redsh) ! i.e. ran uniform

                        endif !                        if(ranboot)then

                        if(uniquez)then
                           do iq=1,nqso
                              redsh(iq)= (z1+z2)/2.0
                           enddo
                        endif

c                        call bootVeron  
c     ---  foll. is new version 19/06/00 - avoids extra noise in randoms
                        if(.not.(ranboot))call bootVeron  
c     --- 


                     endif
                  endif         !      if(dogal)then
               endif            !   if(idirn.eq.2)then
            endif               !if(icheck.ge.2)then

C     --- reconvert to galactic coords ---
c            if(dogal)then       ! galactic coords cut
c               call cutgVeron(fields(1,ifi),fields(2,ifi),
c     _              fields(3,ifi),fields(4,ifi),
c     _              gall,galb)
c            endif

c            nbins=nint(exp(log(2.0) 
c     _           *real(int( log(real(nqso)/2.0)/log(2.0) ))))
c            print*,'ifi,nbins=',ifi,nbins
            ymax=50.0           !1000.0

c     ---- create "z-scrambled" list of redshifts ----
            do iq=1,nqso
               red_scram(iq,1)=redsh(iq)
            enddo
               
            if(.not.(dosubflds))then
c               if(.not.(domoreran))then
c                  call scramble(nqso,red_scram(1,1),red_scram(1,2))
c     elseif(domoreran)then

               if(.not.(samez))then
                  do iran=1,moreran
                     if(test)print*,'iran=',iran,
     _                    ' will call scramble'

                     if(.not.(dosmoodNdz))then
                        call scramble(nqso,red_scram(1,1),
     _                       red_scram((iran-1)*nqso+1,2))               
                     else ! option added 14/06/00
                        call smoothdNdz(nqso, zsmmin,zsm,
     _                       red_scram(1,1),
     _                       red_scram((iran-1)*nqso+1,2))               
                     endif

                     if(test)print*,'iran=',iran,
     _                    ' exited from scramble'
                  enddo
               else
                  do iran=1,moreran
                     do iq=1,nqso
                        iqtot=(iran-1)*nqso+iq
                        red_scram(iqtot,2)=redsh(iq)
                     enddo
                  enddo
               endif

               if(uniform2d)then ! added 06/06/00
                  if(.not.(radivide.and.idirn.eq.2))then
                     print*,'test A'

                     call ranx4Veron(nqso *moreran ,ra_scram,
     _                    (fields(1,ifi)),
     _                    (fields(2,ifi)) )
                  else
                     print*,'test A2'
                     div1= 16.75d0
                     div2= 23.25d0
                     
                     i1=icount8Veron(nqso,ra_, 
     _                    dble(fields(1,ifi)),div1)
                     i2=icount8Veron(nqso,ra_, div1,div2)
                     
                     i3=icount8Veron(nqso,ra_, div2,
     _                    dble(fields(2,ifi)))
                     
                     if(do_cutalpha.and.only1radiv)then
                        call ranx4Veron((i1+i2) *moreran ,
     _                       ra_scram(1),
     _                       (fields(1,ifi)),real(div2))
                     else
                        call ranx4Veron(i1 *moreran ,
     _                       ra_scram(1), 
     _                       (fields(1,ifi)),real(div1))

                        call ranx4Veron(i2 *moreran ,
     _                       ra_scram(i1*moreran+1),
     _                       real(div1),real(div2))
                     endif

                     call ranx4Veron(i3 *moreran ,
     _                    ra_scram((i1+i2)*moreran+1),real(div2),
     _                    (fields(2,ifi)))
c                     call scramble(nqso*moreran, 
c     _                    ra_scram, ra_scram)
                  endif         !(.not.(radivide))then

                  if(.not.(dedivide.and.idirn.eq.3))then
                     print*,'test B'
                     call ranx4Veron(nqso*moreran,decscram,
     _                    (fields(3,ifi)),
     _                    (fields(4,ifi)) )
                     
                  else !if(dedivide)then
                     print*,'test B2'
                     div1= -37.5d0
                     div2= -32.5d0
                     i1=icount8Veron(nqso,dec,
     _                    dble(fields(3,ifi)), div1)
                     i2=icount8Veron(nqso,dec, 
     _                    div1,div2)
                     i3=icount8Veron(nqso,dec, 
     _                    div2,dble(fields(4,ifi)))
                     
                     call ranx4Veron(i1*moreran,decscram(1),
     _                    (fields(3,ifi)), real(div1))
                     call ranx4Veron(i2*moreran,
     _                    decscram(i1*moreran+1),
     _                    real(div1),real(div2))

                     call ranx4Veron(i3*moreran,
     _                    decscram((i1+i2)*moreran+1),
     _                    real(div2),(fields(4,ifi)))

c                     call scramble(nqso*moreran,
c     _                    decscram,decscram)

                  endif         ! (.not.(dedivide))then

               endif            ! if(uniform2d)then ! added 06/06/00

            else                !if((dosubflds))then
               do iran=1,moreran
                  iq=0
                  do jf=1,nfldtot
                     if(.not.(dosmoodNdz))then
                        call scramble(
     _                       npfld(jf),
     _                       red_scram(iq+1,1),
     _                       red_scram((iran-1)*nqso+iq+1,2))
                     else
                        call smoothdNdz(
     _                       npfld(jf), zsmmin,zsm,
     _                       red_scram(iq+1,1),
     _                       red_scram((iran-1)*nqso+iq+1,2))

                     endif


                     iq=iq+npfld(jf)
                  enddo
                  if(iq.ne.nqso)then
                     print*,'plates: iq, nqso=',iq,nqso,
     _                    ' => WARNING !'
                  else
                     print*,'plates: dosubflds check OK iq=nqso=',
     _                    iq,nqso
                  endif
               enddo            !               do iran=1,moreran
            endif            ! if(.not.(dosubflds))then


            do iom=1,nom
               om_0(iom)= om1+real(iom-1)*dom
               do icc=1,ncc
                  cc_0(icc)= cc1+real(icc-1)*dcc
                  if(do_quin)then
                     cc_0flat= 1.0-om_0(iom)
                     cc_0(icc) = ww1+real(icc-1)*dww
                  endif

                  print*,'======='
                  write(6,'(a,2f7.3,$)')
     _                 '=curv params: om_0(iom),cc_0(icc)=',
     _                 om_0(iom),cc_0(icc)
                  if(do_quin)then
                     print*,' cc_0flat, cc_0(icc)=',
     _                    cc_0flat, cc_0(icc)
                  else
                     print*,' '
                  endif


                  q_0=om_0(iom)/2.0 - cc_0(icc)

                  if(do_quin)q_0= om_0(iom)/2.0 - cc_0flat

c                  Rlss=130.0/(H_0/100.0) 
                  Rlss=Rlss0 /(H_0/100.0) 

                  if(.not.(do_quin))then
                     pdWmin= pdWein(q_0,H_0,cc_0(icc), z1)
                  else
                     pdWmin= pdWeiq(om_0(iom),H_0,
     _                    cc_0flat,cc_0(icc), z1)
                  endif

                  
c                  print*,'test AAA A',pdWmin,
c     _                 pdWein(q_0,H_0,cc_0flat, z1)

c                  pdWmin=min(pdWmin, 8.0*Rlss) ! test only for (2.4,3.5)

c                  pdWmin= pdWein(0.5,H_0,0.0, z1) !test 8/11/99 indep of iom,icc

                  do iq=1,nqso
                     if(.not.(do_quin))then
                        pdW(iq)= pdWein(q_0,H_0,cc_0(icc), redsh(iq))
                     else
                        pdW(iq)= pdWeiq(om_0(iom),H_0,cc_0flat,
     _                       cc_0(icc), redsh(iq))
                     endif
                  enddo
                  if(idirn.eq.1)then
                     do iq=1,nqso
                        pdW(iq)=pdW(iq) - pdWmin
                     enddo
                     if(.not.(do_quin))then
                        pdWmax= pdWein(q_0,H_0,cc_0(icc), zmax)
     _                       - pdWmin ! ajout 3/11/99
                     else
                        pdWmax= pdWeiq(om_0(iom),H_0,cc_0flat,
     _                       cc_0(icc), zmax)
     _                    - pdWmin ! ajout 3/11/99
                     endif

                  elseif(idirn.eq.2.or.idirn.eq.3)then
                     curv0= om_0(iom)+cc_0(icc) -1.0

                     if(do_quin)curv0=0.0

                     if(.not.(do2dF))then
                        pdWmax=pdWmin ! 3/11/99
                     else
                        if(.not.(do_quin))then
                           pdWmax=pdWein(q_0,H_0,cc_0(icc), z2) ! 11/05/00
                        else
                           pdWmax=pdWeiq(om_0(iom),H_0,
     _                          cc_0flat,cc_0(icc), z2) ! 19/05/01
                        endif
                     endif

                     if(curv0.lt.-tolcurv)then
                        kcurv=-1
                        rcurv= conh/(H_0/100.0) /sqrt( -curv0 )
                        rcurv8= conh/(H_0/100.0) /sqrt( -curv0 )
                        pdWmax= rcurv*sinh(pdWmax/rcurv) *thetamax
                     elseif(curv0.gt.tolcurv)then
                        kcurv=+1
                        rcurv= conh/(H_0/100.0) /sqrt( curv0 )
                        rcurv8= conh/(H_0/100.0) /sqrt( curv0 )
                        pdWmax= rcurv*sin(pdWmax/rcurv) *thetamax
C     --- following unnecessary if Om, lam < 1 and z < 8 ----
c     _                       rcurv*thetamax)
                     else
                        kcurv=0
                        rcurv=9e9
                        pdWmax=pdWmax*thetamax
                     endif

c                     delta=10.0 ! h^-1 Mpc
                     delta=5.0 ! 2.0 !h^-1 Mpc ! 29/05/00
                     if(kcurv.ne.0)then
                        deltaRc= delta/rcurv
                     endif
c                     nbin(icc,iom)= pdWmax(icc,iom)/delta
                     nbin(icc,iom)= pdWmax/delta

                     if(.not.(docorrn))then
                        do iq=1,nqso
c     dtheta= real(ra_(iq))*pi_180 
                           if(idirn.eq.2)then
                              if(.not.(dogal))then
                                 dtheta=(real(ra_(iq))-fields(1,ifi))
     _                                *pi_180 
     _                                *cos( real(dec(iq))*pi_180 )
                              else
                                 dtheta= ((gall(iq))-fields(1,ifi))
c     dtheta= (fields(2,ifi) -gall(iq))
     _                                *pi_180 
     _                                *cos( (galb(iq))*pi_180 )
                              endif
                           else ! if(idirn.eq.3)then
                              if(.not.(dogal))then
                                 dtheta=(real(dec(iq))-fields(3,ifi))
     _                                *pi_180 
                              else
                                 dtheta= ((gall(iq))-fields(3,ifi))
c     dtheta= (fields(4,ifi) -gall(iq))
     _                                *pi_180 
                              endif
                           endif

                           if(kcurv.lt.0)then
                              pdW(iq)= 
     _                             rcurv*sinh(pdW(iq)/rcurv)*dtheta
                           elseif(kcurv.gt.0)then
                              pdW(iq)= 
     _                             rcurv*sin(pdW(iq)/rcurv)*dtheta
                           else
                              pdW(iq)= pdW(iq)*dtheta
                           endif
                        enddo
                     else
                        
                        do itwo=1,ntwo
c                           if(itwo.eq.2)then
c                              call ranzVeron !! modifies redsh(.) !!
c                           endif

                           if(itwo.eq.1)then
                              nmran=1
                           else
                              nmran=moreran
                           endif

                           if(test)print*,'itwo=',itwo
                           if(test)print*,'mxbigq=',mxbigq
c                           do iq=1,nqso
                           do iran=1,nmran
                              do iq=1,nqso
                                 iqtot=(iran-1)*nqso+iq

c     if(test.and.100*(iq/100).eq.iq)then
c     print*,'iq = ',iq
c     endif

c                                 if(kcurv.ne.0)then
c                                    chid=dble( 
c     _                                   pdWein(q_0,H_0,cc_0(icc),
c     _                                   red_scram(iqtot,itwo)) 
c     _                                   /rcurv)
c                                    pdW_1(iqtot,itwo)= chid*rcurv8
c                                 else
c                                    chid=dble( 
c     _                                   pdWein(q_0,H_0,cc_0(icc),
c     _                                   red_scram(iqtot,itwo)) )
c
c                                    pdW_1(iqtot,itwo)= chid
c                                 endif

                                 if(.not.(do_quin))then
                                    chid=dble( 
     _                                   pdWein(q_0,H_0,cc_0(icc),
     _                                   red_scram(iqtot,itwo)))
                                 else
                                    chid=dble( 
     _                                   pdWeiq(om_0(iom),H_0,
     _                                   cc_0flat,cc_0(icc),
     _                                   red_scram(iqtot,itwo))) 
                                 endif

                                 if(kcurv.ne.0)then
                                    chid=chid/dble(rcurv)
                                    pdW_1(iqtot,itwo)= chid*rcurv8
                                 else
                                    pdW_1(iqtot,itwo)= chid
                                 endif


                                 if(kcurv.lt.0)then
                                    dd=sinhd(chid)
                                    w_1(iqtot,itwo)=coshd(chid)
                                 elseif(kcurv.gt.0)then
                                    dd=sin(chid)
                                    w_1(iqtot,itwo)=cos(chid)
                                 else
                                    dd=chid 
                                    w_1(iqtot,itwo)=0.0
                                 endif
                                 
c     -- added 06/06/00 --
c                                 if(icheck.eq.1.and.itwo.eq.2
c     -- test repeated below !! modify in both cases if modifying !! ----
c     - search on "decd" to find other cases of test .
                                 if(
     _                                itwo.eq.2
     _                                .and.uniform2d)then
                                    decd= decscram(iqtot)
                                    ra_d= ra_scram(iqtot)
                                 else
                                    decd= dec(iq)
                                    ra_d= ra_(iq)
                                 endif
                                 
                                 x_1(iqtot,itwo)=
c     _                                dd *cos(dec(iq)*pi_180) 
c     _                                *cos(ra_(iq)*pi_180)
     _                                dd *cos(decd*pi_180) 
     _                                *cos(ra_d*pi_180)
                                 y_1(iqtot,itwo)=
c     _                                dd *cos(dec(iq)*pi_180)
c     _                                *sin(ra_(iq)*pi_180)
     _                                dd *cos(decd*pi_180)
     _                                *sin(ra_d*pi_180)
                                 z_1(iqtot,itwo)=
c     _                                dd *sin(dec(iq)*pi_180)
     _                                dd *sin(decd*pi_180)
                                 
                              enddo ! do iq=1,nq !so
                           enddo !do iran=1,nmran
                        enddo   ! do itwo=1,ntwo
                        if(test)print*,'End of itwo loop.'

                     endif      !   if(.not.(docorrn))then

                  else
                     stop 'idirn not implemented'
                  endif

                  
c     --- binning ---
                  if(.not.(docorrn))then
                     call mkbins(nqso,0.0,pdWmax, pdW, nbins, 
     _                    r(1), bmin,bmax, dN, ioob)

                     bmax2=bmax

                     if(plotdN)then
                        call pgbegin(0,'?',1,1)
                        call pgvsize(1.0,6.0,1.0,6.0)
                        call pgscf(2)
                        call pgslw(3)

                        ifact=4 !2
                        y2=bmax2*1.05*real(ifact)
                        call pgwindow(0.0,pdWmax,
     _                       0.0,y2)
                        call pgbox('bcnts',0.,0,'bcnts',0.,0)
c     _              call pghist(nqso,redsh,0.0,zmax,nbins,5)  !! debug only !!
                        call pghist(nqso,pdW,0.0, 
     _                       pdWmax,nbins/4/ifact,5)
                        call pglabel('d\\d\\(0738)\\u (h\\u-1\\dMpc)',
     _                       'N',' ')

                        print*,'bin size = ',
     _                       pdWmax/(real(nbins/4/ifact))

                        write(ch5a,'(f5.3)')om_0(iom)
                        write(ch5b,'(f5.3)')cc_0(icc)
                        if(idirn.eq.2.and.(.not.(dogal)))then
                           ch5c='\\ga '
                        elseif(idirn.eq.3.and.(.not.(dogal)))then
                           ch5c='\\gd '
                        else
                           print*,'Warning: label not defined !!'
                        endif

                        call pgptxt(0.05*pdWmax,0.95*y2,0.0,0.0,
     _                       '\\gW\\d0\\u='//ch5a(1:4)//
     _                       ', \\gl\\d0\\u='//ch5b(1:4)//
     _                       ', '//ch5c)


                        call pgend
                     endif

c     Rlss=130.0/(H_0/100.0)
c     --- test only ---
                     if(1+1.eq.3)then
                        do i=1,nbins
                           xx=sin(r(i)*2.0*3.1416/Rlss)*0.5 +0.5
                           xx= xx*xx*xx*xx
                           dN(i)=xx
                        enddo
                     endif

                     do i=1,nbins
                        dNc(i)= dN(i)
                     enddo
                     isign=1
c     call four1(dN8,nbins,isign)
                     call four1(dNc,nbins,isign)
                     bmax2=-9e9

                     do i=1,nbins/2-1
                        rfreq(i)= real(i)/r(nbins)
                        rfreq(-i)= real(-i)/r(nbins)
                        dNfour(i)=abs(dNc(i+1))
                        dNfour(i-nbins/2)= abs(dNc(i+nbins/2+1))
                        bmax2=max(bmax2,dNfour(i),dNfour(i-nbins/2))
                     enddo
                     rfreq(0)=0.0
                     dNfour(0)=abs(dNc(1)) !0.0 ! 
                     bmax2=max(dNfour(0),bmax2)


                     if(plotdN)then
                        call pgbegin(0,'?',1,1)
                        call pgvsize(1.0,6.0,1.0,6.0)
                        call pgscf(2)
                        call pgslw(3)

                        ihighest= nint( 0.01 / rfreq(1) ) ! 16 !
                        xx2= real(ihighest)/pdWmax
                        y2=bmax2*1.05/6.0
                        call pgwindow(0.0,xx2,
     _                       0.0,y2)
                        call pgbox('bcnts',0.,0,'bcnts',0.,0)
                        
                        call pgline(nbins/2,rfreq(0),dNfour(0))

                        call pglabel('1/d\\dpm',
     _                       'f(1/d\\dpm\\u)',' ')

                        write(ch5a,'(f5.3)')om_0(iom)
                        write(ch5b,'(f5.3)')cc_0(icc)
                        if(idirn.eq.2.and.(.not.(dogal)))then
                           ch5c='\\ga '
                        elseif(idirn.eq.3.and.(.not.(dogal)))then
                           ch5c='\\gd '
                        else
                           print*,'Warning: label not defined !!'
                        endif

                        call pgptxt(0.05*xx2,0.95*y2,0.0,0.0,
     _                       '\\gW\\d0\\u='//ch5a(1:4)//
     _                       ', \\gl\\d0\\u='//ch5b(1:4)//
     _                       ', '//ch5c)


                        call pgend
                     endif
                  elseif(docorrn)then
                     if(test)print*,'starting ith loop...'

c                     do iqso=1,nqso-1
c                        do jqso=iqso+1,nqso
c                           isclose(jqso,iqso)=.true. ! default
c                        enddo
c                     enddo

                     do ith=1,nthree
                        do ib=1,nbin(icc,iom)
c                           histog(ib,ith)=0.0
                           hist8(ib,ith)=0.0d0
                        enddo
                        
                        if(test)print*,
     _                       'Starting iom,icc,ith=',iom,icc,ith

                        if(ith.eq.1)then
                           itwo1=1
                           itwo2=1
                        elseif(ith.eq.2)then
                           itwo1=1
                           itwo2=2
                        elseif(ith.eq.3)then
                           itwo1=2
                           itwo2=2
                        endif

                        ip=0 ! number of pairs tested
                        ipp=0 ! number really used (passing all tests)
                        if(ith.eq.1)then
                           iq1=1
                           iq2=nqso-1
                           jq2=nqso
                        elseif(ith.eq.2)then
                           iq1=1
                           iq2=nqso
                           jq1=1
                           jq2=nqso*moreran
                        elseif(ith.eq.3)then
                           iq1=1
                           iq2=nqso*moreran-1
                           jq2=nqso*moreran
                        endif

c                        do iqso=1,nqso-1
c                           do jqso=iqso+1,nqso

                        do iqso=iq1,iq2
                           if(ith.eq.1.or.ith.eq.3)then
cccc     _                          .or.ith.eq.2)then
                              jq1=iqso+1
                           endif
                           do jqso=jq1,jq2
c     print*,iqso,jqso
                              ip=ip+1

c                              if(isclose(jqso,iqso))then
c                              if(1+1.eq.2)then  ! redundant test !
                              if( (
     _                             (.not.(avoidRRcross).or.
     _                             ith.le.2).or.
     _                             (avoidRRcross.and.
     _                             ((iqso-1)/nqso.eq.
     _                             (jqso-1)/nqso))
     _                             ) .and.
     _                             (   (.not.(doeqzs)) .or.
     _                             (doeqzs.and.
     _                             ( abs(red_scram(iqso,itwo1) 
     _                             -red_scram(jqso,itwo2))
     _                             .lt. dz_equal ))  )
     _                             )then

                                 xx=abs(
     _                                w_1(iqso,itwo1)-
     _                                w_1(jqso,itwo2))+abs(
     _                                x_1(iqso,itwo1)-
     _                                x_1(jqso,itwo2))+abs(
     _                                y_1(iqso,itwo1)-
     _                                y_1(jqso,itwo2))+abs(
     _                                z_1(iqso,itwo1)-
     _                                z_1(jqso,itwo2))

                                 if(kcurv.eq.0)then
                                    xx=xx/delta
                                 else
                                    xx=xx/deltaRc
                                 endif

                                 if(do_radtan)then
                                    if(itwo1.eq.2
     _                                   .and.uniform2d)then
                                       decdi= decscram(iqso)
                                       ra_di= ra_scram(iqso)
                                    else
                                       decdi= dec(iqso)
                                       ra_di= ra_(iqso)
                                    endif

                                    if(itwo2.eq.2
     _                                   .and.uniform2d)then
                                       decdj= decscram(jqso)
                                       ra_dj= ra_scram(jqso)
                                    else
                                       decdj= dec(jqso)
                                       ra_dj= ra_(jqso)
                                    endif

c                                    pdi= pdWein(q_0,H_0,cc_0(icc),
c     _                                   red_scram(iqso,itwo1))
c                                    pdj= pdWein(q_0,H_0,cc_0(icc),
c     _                                   red_scram(jqso,itwo2))
                                    pdi= pdW_1(iqso,itwo1)
                                    pdj= pdW_1(jqso,itwo2)

                                    dprad= abs(pdj-pdi)
                                    pdmean= (pdj+pdi)*0.5
                                    
                                    if(pythagOK)then
                                       d_ra_=(ra_di-ra_dj)
     _                                      *cos( (decdi+decdj)*0.5d0
     _                                      *pid_180 )
                                       d_dec=(decdi-decdj)

                                       dthe_d= sqrt(d_ra_*d_ra_
     _                                      + d_dec*d_dec)*pid_180
                                    else
                                       stop 'pythagOK=.false. !!'
                                    endif

                                    if(kcurv.lt.0)then
                                       dptan= 
     _                                      rcurv8*
     _                                      sinhd(pdmean/rcurv8)*
     _                                      dthe_d
                                    elseif(kcurv.gt.0)then
                                       dptan= 
     _                                      rcurv8*
     _                                      sin(pdmean/rcurv8)*
     _                                      dthe_d
                                    else
                                       dptan= pdmean*dthe_d
                                    endif

                                    if(dptan.lt.tol_dptan)
     _                                   dptan=tol_dptan
                                    radtantest= 
     _                                   (radvstan.and.
     _                                   dprad/dptan.gt.rtrat8).or.
     _                                   (.not.(radvstan).and.
     _                                   dprad/dptan.lt.rtrat8)

                                 endif

                                 if(xx.gt.tolerdd.and.
c     -- avoid having to do the calculation if radtantest=.false. --
     _                                ( (.not.(do_radtan)).or.
     _                                (do_radtan.and.radtantest) )
     _                                )then
                                    dd=angle4D8(kcurv,
     _                                   w_1(iqso,itwo1),
     _                                   x_1(iqso,itwo1),
     _                                   y_1(iqso,itwo1),
     _                                   z_1(iqso,itwo1),
     _                                   w_1(jqso,itwo2),
     _                                   x_1(jqso,itwo2),
     _                                   y_1(jqso,itwo2),
     _                                   z_1(jqso,itwo2) )
                                 else
                                    dd=0.0
                                 endif
c     pair(ip)=real(dd)
                                 
                                 if( (.not.(do_radtan)).or.
     _                                (do_radtan.and.radtantest) )
     _                                then

                                    if(dd.lt.-99.0)then
                                       print*,'iqso,jqso=',iqso,jqso
                                    endif

                                    if(kcurv.ne.0)then
c     ib= int( dd/deltaRc + 0.5 )
                                       ib= int( dd/deltaRc )
                                    else
c     ib= int( dd/delta + 0.5 ) ! swapped out 22/06/00 (above swapped earlier)
                                       ib= int( dd/delta )
                                    endif
                                    if(ib.lt.1)then
                                       if(iwarn1.gt.0)then
                                          print*,'"fpairs" WARNING:',
     _                                         ' ith,iqso,jqso,dd,ib'
                                          print*,ith,iqso,jqso,dd,ib
                                          iwarn1=iwarn1-1
                                       endif
                                    elseif(ib.gt.nbin(icc,iom))then
                                       if(iwarn2.gt.0)then
                                          print*,'"fpairs" WARNING:',
     _                                         ' ith,iqso,jqso,dd,ib'
                                          print*,ith,iqso,jqso,dd,ib
                                          iwarn2=iwarn2-1
                                       endif
                                    else
c     histog(ib,ith)=histog(ib,ith)+1.0
                                       hist8(ib,ith)=hist8(ib,ith)+1.0d0
                                       ipp=ipp+1
                                    endif

c                                    ipp=ipp+1 ! correct position? 18/07/00

                                 endif! if( (.not.(do_radtan)).or. ...


                              endif !   if(isclose(jqso,iqso))then

                           enddo !do jqso=iqso+1,nqso
                        enddo   !do iqso=1,nqso-1
                        np=ip
c                        if(test)print*,'ith=',ith,
                        if(ith.eq.1)then
                           print*,
     _                          'nqso*(nqso-1)/2=',nqso*(nqso-1)/2
                        elseif(ith.eq.2)then
                           print*,
     _                          'nqso*nqso*moreran=',
     _                          nqso*nqso*moreran
                        elseif(ith.eq.3)then
                           print*,
     _                          '(nqso*moreran)'//
     _                          '*(nqso*moreran-1)/2=',
     _                          (nqso*moreran)*(nqso*moreran-1)/2
                        endif

                        paircount(ith)=dble(ipp)

                        if(1+1.eq.2)print*,'ith=',ith,
     _                       ' done pair list, np=',np,'ipp=',ipp

                     enddo !                     do ith=1,nthree
                     print*,'Done histograms, icheck=',icheck,
     _                    '  np=',np
                     
                     if(.not.(do_radtan)
     _                    .and.(.not.(avoidRRcross)))then
                        fmore1_8= 2.0d0* dble(moreran)*dble(nqso)/
     _                       dble(nqso-1)
                        fmore2_8= dble(moreran)
     _                       *dble(nqso*moreran-1)/
     _                       dble(nqso-1)
c     fmore1=real(fmore1_8)
c     fmore2=real(fmore2_8)
                     elseif(do_radtan.or.avoidRRcross)then
                        if(paircount(1).lt.0.99)print*,
     _                       'ERROR: paircount(1)=',paircount(1)
                        paircount(1)=max(paircount(1),0.99d0)

                        fmore1_8=paircount(2)/paircount(1)
                        fmore2_8=paircount(3)/paircount(1)
                     endif

                     print*,'fmore1_8,fmore2_8=',fmore1_8,fmore2_8




                     do ib=1,nbin(icc,iom)
c                        if(histog(ib,3).gt.0.0)then
c                        if(hist8(ib,3).gt.0.0)then
                        if(hist8(ib,3).gt.0.0
     _                       .and.hist8(ib,2).gt.0.0
     _                       )then
                           corrn(ib)= 
c     --- Landy & Szalay 1993 ---
c     _                          (histog(ib,1)-2.0*histog(ib,2))  
c     _                          /histog(ib,3) +1.0
c     -- In the updated formula below, 
c     (i) the factor of 2.0 could be dropped
c     because there are twice as many distinct cross-correlated pairs
c     as auto-correlated pairs (d_ij = d_ji for auto correlation,
c     but not for cross-correlation), but ...
c     and (ii) the factors fmore are
c     simply to account for having more randoms than data points. --
c
c     _                          (histog(ib,1)-histog(ib,2)/fmore1)  
c     _                          /(histog(ib,3)/fmore2) +1.0
c     LS 1993
c     19/06/00 : this seems correct (cf DD/RR-1, DD/DR-1 for fmore1,fmore2) --
     _                          real( 
     _                          (hist8(ib,1)
     _                          -2.0d0* hist8(ib,2)/fmore1_8)  
     _                          /(hist8(ib,3)/fmore2_8) +1.0d0   )
c     "nave" formula (RP94) DD/RR - 1
c     _                          real( 
c     _                          hist8(ib,1)
c     _                          /(hist8(ib,3)/fmore2_8) -1.0d0 )
c     DD/DR -1 
c     _                          real( 
c     _                          hist8(ib,1)
c     _                          /(hist8(ib,2)/fmore1_8) -1.0d0 )

                           do jth=1,3
                              histog(ib,jth)=hist8(ib,jth)
                           enddo

                        else
                           corrn(ib)=-99.9
                        endif
                     enddo

                     if(icheck.eq.1.and.iom.eq.1.and.icc.eq.1)then
                        open(1,file='corrn.dat',access='sequential',
     _                       form='unformatted',status='unknown')
                        write(1)iom,icc,om_0(iom),cc_0(icc),
     _                       nbin(icc,iom),kcurv,delta,deltaRc
                        write(1)nbin,histog,corrn
                        close(1)
                        print*,'Have written '//'corrn.dat'
                     endif
                  endif ! if(.not.(docorrn))then

c     ==== extract statistics ----

                  if(.not.(docorrn))then
c     i1=8
                     if(idirn.eq.1)then
                        ifind1=8
                        nsearch=nbins/2-1-ifind1
                        ismoo=2
                     elseif(idirn.eq.2.or.idirn.eq.3)then
                        freqmax= 0.01
c     ifind1=3  ! for the +47deg survey
                        ifind1=2
C     ---- rough expression for first approximation ----
c     nsearch= int( 
c     _                    thetamax/2.0/pi *real(nbins/2-1) ) -ifind1 

c     nsearch=14 ! for the +47deg survey
c     nsearch=nbins/2-1-ifind1
                        
                        nsearch=nint( freqmax/rfreq(1) ) 
                        
                        nsearch=max(1,min(nsearch,nbins/2-1-ifind1))
c     ismoo=1 ! for the +47deg survey
c     ismoo=nint( 10.0 ! added 3/11/99
c     _                    *(thetamax/6.283) )
c     ismoo=5 ! for the 10<b<20 analysis

                        ismoo=2 !3 !
c     print*,'nsearch=',nsearch

c     irebin=4 
                        irebin=nint(3.0 * thetamax/6.283) +1 ! added 3/11/99
                        irebin=irebin+1

                        irebin=0
c     irebin=5 ! just for plotting !!!
c     call rebin(nsearch+ifind1,dNfour(i1), irebin)

                     else
                        stop 'option undefined'
                     endif
                     i1= ifind1


c     call findmin(nbins/2-1-i1,dNfour(i1), ismoo, 
                     call findmin(nsearch,dNfour(i1), ismoo, 
     _                    imin1,imax2,imax3,imaxfin, modif_y)
                     imin1=imin1+i1-1
                     imax2=imax2+i1-1
                     imax3=imax3+i1-1
                     imaxfin=imaxfin+i1-1
                     

c     print*,'imin1,rfreq(imin1),dNfour(imin1)='
c     print*,imin1,rfreq(imin1),dNfour(imin1)

c     print*,'om,cc=',om_0(iom),cc_0(icc)
c     print*,'imax2,rfreq(imax2),dNfour(imax2)='
c     print*,imax2,rfreq(imax2),dNfour(imax2)
c     print*,'imaxfin,rfreq(imaxfin),dNfour(imaxfin)='
c     print*,imaxfin,rfreq(imaxfin),dNfour(imaxfin)
                     

                     ib=icheck
                     diffstat(icc,iom,1,ib)= rfreq(imax2) ! frequency

c     diffstat(icc,iom,2,ib)= dNfour(imax2) ! height
                     heismoo= 0.25*dNfour(imax2-1)
     _                    + 0.5*dNfour(imax2)
     _                    + 0.25*dNfour(imax2+1)
                     diffstat(icc,iom,2,ib) = heismoo

                     diffstat(icc,iom,3,ib)= rfreq(imaxfin) ! frequency
c     diffstat(icc,iom,4,ib)= dNfour(imaxfin) ! height
                     heismoo= 0.25*dNfour(imaxfin-1)
     _                    + 0.5*dNfour(imaxfin)
     _                    + 0.25*dNfour(imaxfin+1)

c     diffstat(icc,iom,4,ib) = heismoo ! modified 5/11/99
                     
                     ifreq= nint( 1.0/Rlss / rfreq(1) )
                     if(ifreq.le.2.or.ifreq.ge.nsearch)print*,
     _                    'WARNING: ifreq=',ifreq
                     ifreq=max(2,min(ifreq,nsearch))

                     heismoo= 0.25*dNfour(ifreq-1)
     _                    + 0.5*dNfour(ifreq)
     _                    + 0.25*dNfour(ifreq+1)
                     diffstat(icc,iom,4,ib) =  dNfour(ifreq) ! unsmoothed
c     diffstat(icc,iom,4,ib) =  heismoo ! added 10/11/99

                     if(rfreq(imaxfin).gt.0.0)then
                        xx= rfreq(imaxfin)/rfreq(imax2) -2.0
c     yy= xx*xx
                        yy=xx
                        diffstat(icc,iom,5,ib)= yy
c     else
c     print*,'WARNING: rfreq(imaxfin)=',rfreq(imaxfin)
                     endif



c     if(1+1.eq.2)then
                     if(test)then
c     bmax2=min(bmax2,2000.0)
                        bmax2=min(bmax2,200.0)
                        call pgbegin(0,'?',1,1)
                        call pgvsize(1.0,6.0,1.0,6.0)
                        call pgscf(2)
                        call pgslw(3)

                        call pgwindow(-0.01,0.01,
     _                       0.0,bmax2*1.05)
                        call pgbox('bcnts',0.,0,'bcnts',0.,0)

c     call pgline(nbins,r(-nbins/2),dNfour(-nbins/2))
c     call pgline(nbins,r,dNfour)
                        call pgline(nbins-1,
     _                       rfreq(-nbins/2+1),dNfour(-nbins/2+1))

c     call pgline(nbins,rmp,dNfour)

                        call pglabel('1/d (h\\u-1\\dMpc)\\u-1\\d',
     _                       'N',' ')

                        call pgmove(1.0/Rlss,0.0)
                        call pgdraw( 1.0/Rlss,bmax2*1.05)
                        call pgsls(2)
                        call pgmove(0.5/Rlss,0.0)
                        call pgdraw(0.5/Rlss,bmax2*1.05)
c     call pgsls(4)
c     call pgmove(1.0/(3.0*Rlss),0.0)
c     call pgdraw(1.0/(3.0*Rlss),bmax2*1.05)
                        call pgsls(1)

                        call pgwindow(0.0,1.0,0.0,1.0) !! warning: window resized
                        call pgsch(1.2)

                        write(ch3,'(i3)')ifi
                        if(icheck.eq.1)then
                           call pgptxt(0.05,0.95,0.0,0.0,
     _                          ch3//": correct z's")
                        elseif(icheck.ge.2)then
                           call pgptxt(0.05,0.95,0.0,0.0,
     _                          ch3//": randomised z's")
                        endif
                        write(ch5a,'(f5.3)')om_0(iom)
                        write(ch5b,'(f5.3)')cc_0(icc)
                        call pgptxt(0.05,0.85,0.0,0.0,
     _                       '\\gW\\d0\\u='//ch5a(1:4)//
     _                       ', \\gl\\d0\\u='//ch5b(1:4))



                        call pgend
                     endif
                  elseif(docorrn)then
                     n= 60 !150! 30 !11240! 
                     if(doRLAGN)n=1000
c                     i1= 5! 7 !8
                     i1= 10

                     i1_sm= 1! i1
                     nb_sm= i1+n 

                     ismoo=1 !0
                     ifreq= Rlss/delta

c                     test=.true.

c     --- reverse order corrn(i1...) so that imaxfin is first max ---
                     ib=icheck
                     do i=i1,i1+n-1
                        corrn_cp(i1+n-i)=corrn(i)
c                        if(n.le.mxb_sm)then
c                           corrnall(icc,iom, i-i1+1, ib)=corrn(i)
c                        endif
                     enddo

                     if(i1+n.gt.mxb_sm)then
                        print*,'WARNING: i1+n, mxb_sm=',
     _                       i1+n,mxb_sm,' corrnall problem !'
                     endif
                     do i=1,min(i1+n, mxb_sm)
                        corrnall(icc,iom, i, ib)=corrn(i)
                     enddo


c                     call findmin(n,corrn(i1),ismoo, 
                     call findmin(n,corrn_cp,ismoo, 
     _                    imin1,imax2,imax3,imaxfin, modif_y)

c                     imaxfin=imaxfin+i1-1
                     imaxfin=(n+1-imaxfin)+(i1-1)
                     
                     ib=icheck
                     diffstat(icc,iom,3,ib)= 
     _                    1.0/(delta*(real(imaxfin)-0.5)) ! frequency
                     diffstat(icc,iom,4,ib)= corrn(ifreq) ! amplitude



c     ---- save "full" correlation information ----
c                     if(icheck.eq.mxboot
c     _                    .and.iom.eq.nom.and.icc.eq.ncc)then
                     if(icheck.eq.mxboot
     _                    .and.((iom.eq.nom.and.icc.eq.ncc).or.
     _                    ( (4*(iom/4).eq.iom.and.
     _                    4*(icc/4).eq.icc) )) 
     _                    )then
                        write(ch,'(i1)')i2dF

                        ch='s' !'u' !! test only !! (for bg jobs)

                        open(1,file='corrnall'//ch//'.dat',
     _                       access='sequential',
     _                       form='unformatted',status='unknown')
                        write(1)nom,ncc,(om_0(i),i=1,nom),
     _                       (cc_0(i),i=1,ncc)
                        write(1)i1_sm,nb_sm,delta ! added 29/05/00
c                        write(1)nb_sm,corrnall
                        write(1)nb_sm,((((
     _                       corrnall(icc2,iom2,ib,ich),
     _                       icc2=1,ncc),iom2=1,nom),ib=1,nb_sm),
     _                       ich=1,mxboot)

c     --- "thirdformat" (cf plotcorrnall.f) ---
                        write(1)( (nbin(icc2,iom2),icc2=1,ncc), !added27/09/00
     _                       iom2=1,nom )

                        close(1)
                        print*,'Have written '//
     _                       'corrnall'//ch//'.dat'
                     endif



                  endif         !if(.not.(docorrn))then

               enddo            !do icc=1,ncc
            enddo               !do iom=1,nom


            transp=.false.
c     --- is istat OK here ??? ---
            if(1+1.eq.3)then
               call findAB(diffstat(1,1,1,ib),transp,istat, 
     _              cc_0,om_0,
     _              ncc,nom, Aa,sigAa,Bb,sigBb,chi2,q)
               
               print*,'icheck, Aa,sigAa,Bb,sigBb,chi2,',
     _              'chi2/real(ncc),q='
               print*,icheck, Aa,sigAa,Bb,sigBb,chi2,chi2/real(ncc),q

c     pause
               A(icheck)=Aa
               Asig(icheck)=sigAa
               B(icheck)=Bb
               Bsig(icheck)=sigBb
               chi2n(icheck)=chi2/real(ncc)
            endif
            

            do i=-nbins/2,nbins/2
               if(icheck.eq.1)then
                  dNf_all(i,1)=dNf_all(i,1)+ dNfour(i) ! real
               elseif(2.le.icheck.and.icheck.le.1+nctrl)then
                  dNf_all(i,2)=dNf_all(i,2)+ dNfour(i) ! boot
                  dNf_all(i,3)=dNf_all(i,3)+ dNfour(i)*dNfour(i) !boot
               else !if(nctrl+2.le.icheck.and.icheck.le.1+2*nctrl)then
                  dNf_all(i,4)=dNf_all(i,4)+ dNfour(i) ! poisson
                  dNf_all(i,5)=dNf_all(i,5)+ dNfour(i)*dNfour(i) !poisson
               endif
            enddo

            if(icheck.eq.-1)then
               open(1,file='plates_obs.dat',access='sequential',
     _              form='unformatted',status='unknown')
               write(1)mxcurv,mxcurv,nom,ncc,om_0,cc_0,diffstat
               close(1)
            elseif(icheck.eq.-2)then
               open(1,file='plates_ran.dat',access='sequential',
     _              form='unformatted',status='unknown')
               write(1)mxcurv,mxcurv,nom,ncc,om_0,cc_0,diffstat
               close(1)
            endif

            if(icheck.eq.mxboot)then
               open(1,file='plates.dat',access='sequential',
     _              form='unformatted',status='unknown')
               write(1)mxcurv,mxcurv,nom,ncc,om_0,cc_0,diffstat
               close(1)
            endif

         enddo !         do icheck=1,mxboot

         if(docorrn)then
            print*,'Line fitting not yet programmed for corrn !!!'
            print*,'Please modify plates.f if you want this...'
            stop
         endif


         Am=0.0
         Amsig=0.0
         Am_w2=0.0
         Bm=0.0
         Bmsig=0.0
         Bm_w2=0.0 

         chitot=0.0
         chi_2= 0.0
c         stop 'test A'
         
c         do icheck=1+1,nctrl !mxboot
         do icheck=1+1,1+nctrl !mxboot !changed from nctrl-> 1+nctrl 03/04/00
c            wt= min(1.0, 1.0/ chi2n(icheck))
c            wt= 1.0/ chi2n(icheck)
            wt=1.0
            Am=Am+A(icheck) *wt
            Am_w2=Am_w2 + A(icheck) *wt*wt
c            Amsig=Amsig+Asig(icheck)*Asig(icheck)
            Amsig=Amsig+A(icheck)*A(icheck)  *wt *wt

            Bm=Bm+B(icheck)  *wt
            Bm_w2=Bm_w2 + B(icheck) *wt*wt
c            Bmsig=Bmsig+Bsig(icheck)*Bsig(icheck)
            Bmsig=Bmsig+B(icheck)*B(icheck) *wt *wt

            chitot= chitot+ wt
            chi_2= chi_2+ wt*wt
         enddo
c         stop 'test B'

c         chi_2 = chitot  ! avoiding  wt*wt

c         chitot=real(mxboot-1)
c         chi_2=real(mxboot-1)

         tol=1e-4
         if(chitot.gt.1.0+tol)then
            Am=Am/chitot
            Bm=Bm/chitot
         else
            print*,'Warning: chitot=',chitot,' no statistics on A,B.'
         endif

         if(chi_2.gt.2.0+tol)then
            Am_w2= Am_w2/chi_2

            xx= Amsig/chi_2 - 2.0*Am*Am_w2 + Am*Am 
            if(xx.gt.tol)then
               Amsig= sqrt( xx )
            else
               print*,'WARNING: Amsig^2 =',xx
               if(Amsig.lt.-tol)then
                  Amsig=-sqrt( -xx )
               else
                  Amsig=tol /sqrt(chi_2/(chi_2-1.0))
               endif
            endif

            Bm_w2= Bm_w2/chi_2
            xx= Bmsig/chi_2 - 2.0*Bm*Bm_w2 + Bm*Bm 
            if(xx.gt.tol)then
               Bmsig= sqrt( xx )
            else
               print*,'WARNING: Bmsig^2 =',xx
               if(Bmsig.lt.-tol)then
                  Bmsig=-sqrt( -xx )
               else
                  Bmsig=tol /sqrt(chi_2/(chi_2-1.0))
               endif
            endif

c            Bmsig= sqrt( Bmsig/chi_2 - Bm*Bm ) 

            Amsig=Amsig
     _           *sqrt(chi_2/(chi_2-1.0))
            Bmsig=Bmsig
     _           *sqrt(chi_2/(chi_2-1.0))
         else
            print*,'Warning small total Sum(chi2n): chi_2 = ',chi_2
         endif

         print*,' '
         print*,'chitot,chi_2=',chitot,chi_2
         do i=1+1,mxboot
            write(6,'(a,i,4f8.3,f12.1)')
     _           'i= ',i,A(i),Asig(i),B(i),Bsig(i),chi2n(i)
         enddo

         print*,'Results: '
         print*,'A(1),Am,Amsig, B(1),Bm,Bmsig='
         print*,A(1),Am,Amsig,'  ', B(1),Bm,Bmsig


      enddo!     do ifi=1,nfields

c      stop 'test C'

      do i=-nbins/2,nbins/2
         dNf_all(i,2)= dNf_all(i,2)/real(nctrl)
         dNf_all(i,3)= dNf_all(i,3)/real(nctrl) 
     _        - dNf_all(i,2)*dNf_all(i,2)
         if(dNf_all(i,3).lt.tol)then
            print*,'dNf_all(i,3).lt.tol=',i,dNf_all(i,3),tol
            dNf_all(i,3)= sqrt(tol)
         else
            dNf_all(i,3)= sqrt(dNf_all(i,3))
         endif
         dNf_all(i,4)= dNf_all(i,4)/real(nctrl)
         dNf_all(i,5)= dNf_all(i,5)/real(nctrl) 
     _        - dNf_all(i,4)*dNf_all(i,4)
         if(dNf_all(i,5).lt.tol)then
            print*,'dNf_all(i,5).lt.tol=',i,dNf_all(i,5),tol
            dNf_all(i,5)= sqrt(tol)
         else
            dNf_all(i,5)= sqrt(dNf_all(i,5))
         endif
         dNf_all(i,6)=dNf_all(i,4)+dNf_all(i,5)
         dNf_all(i,7)=dNf_all(i,4)-dNf_all(i,5)
      enddo
      
      bmax3=-9e9
      do i=-nbins/2,nbins/2
         bmax3=max(bmax3,dNf_all(i,1),dNf_all(i,6),dNf_all(i,7))
      enddo

      print*,'bmax3=',bmax3
      bmax3=100.0 !600.0 !20.0 !60.0 !
      iom=nom
      icc=ncc

c     ---- This bit added 03/04/00 to store fourier curves for 
c     different (z1,z2, alpha vs delta) subsets, for analysis
c     as requested by Gary Mamon. ----
c      if(icheck.eq.1)then
         write(chfour(1:10),'(f4.2,a1,f4.2,a1)')
     _        z1,'_',z2,'_'
         if(idirn.eq.2)then
            chfour(11:13)='alp'
         else
            chfour(11:13)='del'
         endif
         open(2,file=chfour,access='sequential',
     _        form='unformatted',status='unknown')
         write(2)iom,icc,om_0(iom),cc_0(icc),z1,z2,
     _        idirn,nbins
c     write(2)(dNfour(i),i=0,nbins/2)
c         write(2)(rfreq(i),i=-nbins/2+1,nbins/2) ! test diff line #s
c         write(2)(dNfour(i),i=-nbins/2+1,nbins/2)! test diff line #s
         write(2)(rfreq(i),i=-nbins/2+1,-nbins/2+nbins)
         write(2)(dNf_all(i,1),i=-nbins/2+1,-nbins/2+nbins) ! signal
         write(2)(dNf_all(i,4),i=-nbins/2+1,-nbins/2+nbins) ! mean Poiss
         write(2)(dNf_all(i,6),i=-nbins/2+1,-nbins/2+nbins) ! mean P +sig
         write(2)(dNf_all(i,7),i=-nbins/2+1,-nbins/2+nbins) ! mean P -sig
         close(2)
c      endif


      call pgbegin(0,'plot'//chfour//'.ps/vps',1,1)
c      call pgbegin(0,'?',1,1)
      call pgvsize(1.0,6.0,1.0,6.0)
      call pgscf(2)
      call pgslw(3)
c      call pgwindow(-0.01,0.01,
      call pgwindow(0.00,0.01,
     _              0.0,bmax3*1.05)
      call pgbox('bcnts',0.,0,'bcnts',0.,0)

      call pgsls(1)
      call pgline(nbins-1,rfreq(-nbins/2+1),dNf_all(-nbins/2+1,1))

      call pgsls(2)
      call pgline(nbins-1,rfreq(-nbins/2+1),dNf_all(-nbins/2+1,4)) !mean

      call pgsls(4)
      call pgline(nbins-1,rfreq(-nbins/2+1),dNf_all(-nbins/2+1,6)) ! m+sig
      call pgline(nbins-1,rfreq(-nbins/2+1),dNf_all(-nbins/2+1,7)) ! m-sig

      call pglabel('1/d (h\\u-1\\dMpc)\\u-1\\d','f',' ')

      call pgsls(1)
      call pgmove(1.0/Rlss,0.0)
      call pgdraw( 1.0/Rlss,bmax3*1.05)
      call pgsls(2)
      call pgmove(0.5/Rlss,0.0)
      call pgdraw(0.5/Rlss,bmax3*1.05)
      call pgsls(1)

      call pgwindow(0.0,1.0,0.0,1.0) !! warning: window resized
      call pgsch(1.2)

      write(ch5a,'(f5.3)')om_0(iom)
      write(ch5b,'(f5.3)')cc_0(icc)
      call pgptxt(0.05,0.95,0.0,0.0,
     _     '\\gW\\d0\\u='//ch5a(1:4)//', \\gl\\d0\\u='//ch5b(1:4))


      call pgend

      end

C      call pgbegin(0,'?',1,1)
C      call pgvsize(1.0,6.0,1.0,6.0)
C      call pgscf(2)
C      call pgslw(3)
C      call pgwindow(0.0,zmax,0.0,ymax)
C      call pgbox('bcnts',0.,0,'bcnts',0.,0)
C      call pghist(nqso,redsh,0.0,zmax,nbins,5)
C      call pglabel('z','N',' ')
C      call pgend

      
