 
        real*4  function gauss(mean,sigma,seed)
c       chooses a number from a gaussian probability distribution
c       given randomising function  rnd(seed)

        integer*4 seed
        real*4    mean,sigma,p
        sigmax=4

        z=1.0
        p=0.0
        do 11 while(z.gt.p)
           x=rnd(seed)
           y=mean*1.0 + 2.0*(x-0.5)*sigma*sigmax
           z=rnd(seed)
           p=exp(-(y-mean)*(y-mean)/2.0/sigma/sigma)
11      continue
        gauss=y
        return
        end

        real*4  function gauss0(mean,sigma)
c       chooses a number from a gaussian probability distribution
c       based on randt()  which can be flushed if desired.

        real*4    mean,sigma,p
        sigmax=4

        z=1.0
        p=0.0
        do 11 while(z.gt.p)
           x= randt()
           y=mean*1.0 + 2.0*(x-0.5)*sigma*sigmax
           z=randt()
           p=exp(-(y-mean)*(y-mean)/2.0/sigma/sigma)
11      continue
        gauss0=y
        return
        end

        function ipoisson(mu,seed)
c       chooses a number from a Poisson probability distribution
c       given randomising function  rnd(seed)

        integer*4 ipoisson,seed
        real*4    mu,sigma,p
        sigmax=4

c       ==== a gaussian approximation suffices and is easier for
c                              large values of  mu:
        if (mu.ge.20) then
            ipoisson=nint(gauss(mu,sqrt(mu*1.0),seed))
            return
            end if

c       ==== a genuine Poisson distribution ========
        sigma=sqrt(mu*1.0)
        z=1.0
        p=0.0
        do 71 while(z.gt.p)
            x=rnd(seed)
            y=int(x *( mu+ sigmax*sigma))
            z=rnd(seed)
c           ---- assign  p= mu^y exp(-mu) / y!
            p=exp(-1.0*mu)
            do 75 i=1,y
               p=p*mu /(float(i))
75         continue
        if (p.gt.1.0) write(6,*)'ERROR: Poisson prob. = ',p
71      continue

        ipoisson=y
        return
        end


        function rnd(seed)
c       generates a number chosen from a uniform probability
c       distribution over  [0,1)  by a prime number clock arithmetic
c       method

        real*4     rnd
        integer*4  seed
c        double precision seed
        integer base,fact,base2
        base=6869
        fact=97
        base2=6871

c        seed=mod(fact*seed,base)
c        rnd=(1.0*seed)/base2
c        call rnd2(randt,seed)
        rnd=randt()
c        rnd=rnd3(seed)
c        rnd=rnd4(seed)


        return
        end

      character*8 function rndname()
      rndname='ranARL'
      return
      end



        function RANDT()
c
c the disadvantage of this program is that you always get the same
c random numbers because you can't seed it. I get around this by having
c a verson that saves the array to a file and then reads the array from the
c file next time the program is run, so in effect this subroutine just carries
c on from where it left off.
c

c
C THIS FORTRAN-CALLABLE FUNCTION RETURNS A PSEUDO-RANDOM REAL
C NUMBER UNIFORMLY DISTRIBUTED OVER THE INTERVAL [0,1].
C CODED IN DECEMBER, 1982 BY
C       W A ALFORD,
C       SCHOOL COMPUTER UNIT,
C       RESEARCH SCHOOL OF PHYSICAL SCIENCES,
C       AUSTRALIAN NATIONAL UNIVERSITY,
C       CANBERRA, ACT
C
C THE FORTRAN CALL TO THE REAL  FUNCTION RANDT IS OF THE FORM:
C
C               Y = RANDT ()
C
C THE CODING IS BASED ON THE FOLLOWING PAPER:
C
C TOOTILL J.P., ROBINSON W.D. AND EAGLE J., 1973. "AN ASYMPTOTICALLY
C       RANDOM TAUSWORTHE SEQUENCE." J.A.C.M. 20(3), 469-481.
C
C IN THE ABSTRACT TO THIS PAPER THEY MAKE THE FOLLOWING STATEMENT.
C
C "AN ASYMPTOTICALLY RANDOM 23-BIT NUMBER SEQUENCE OF ASTRONOMIC PERIOD
C  IS PRESENTED. AN INITIALIZATION PROGRAM IS REQUIRED TO PROVIDE 607 ST
C  VALUES, AFTER WHICH THE SEQUENCE CAN BE GENERATED FROM A THREE-TERM
C  RECURRENCE OF THE FIBONACCI TYPE. IN ADDITION TO POSSESSING THE
C  THEORETICALLY DEMONSTRABLE RANDOMNESS PROPERTIES ASSOCIATED WITH TAUS
C  SEQUENCES, THE SEQUENCE POSSESSES EQUIDISTRIBUTION AND MULTIDIMENSION
C  UNIFORMITY PROPERTIES VASTLY IN EXCESS OF ANYTHING THAT HAS BEEN SHOW
C  CONVENTIONAL CONGRUENTIALLY GENERATED SEQUENCES... THE CLAIMED RANDOM
C  PROPERTIES DO NOT NECESSARILY EXTEND TO SUBSEQUENCES, THOUGH IT IS NO
C  YET KNOWN WHICH PARTICULAR SUBSEQUENCES ARE AT FAULT. ACCORDINGLY, TH
C  SEQUENCE IS AT PRESENT SUGGESTED ONLY FOR SIMULATIONS WITH NO FIXED
C  DIMENSIONALITY REQUIREMENTS."
C
C SOME STATISTICAL TESTS (FREQUENCY, SERIAL, AUTOCORRELATION, GAP AND RU
C TESTS) ON THE OUTPUT FROM THIS CODING ENGENDER CONFIDENCE IN THESE ASS
c AND SHOW THAT THIS PSEUDO-RANDOM NUMBER GENERATOR IS BETTER THAN A
C GENERALIZED FIBONACCI SEQUENCE TYPE GENERATOR CODED BY R BRENT IN USE
C AUSTRALIAN NATIONAL UNIVERSITY. IN TERMS OF SPEED ON AVERAGE THIS GENE
C IS COMPARABLE TO THE LINEAR CONGRUENTIAL AND GENERALIZED FIBONACCI SEQ
C TYPE GENERATORS.
C
C THE INITIALLIZATION OF ARRAY R COMES FROM SUBROUTINE INIT23 IN THE ABO
C PAPER. THE STATISTICAL PROPERTIES OF THE OUTPUT FROM THIS CODING IS HI
C DEPENDENT UPON THE INITIAL SET OF 19 ARBITRARY 32-BIT INTEGERS USED IN
C SUBROUTINE INIT23. IN THIS CASE THEY WERE DERIVED FROM A GENERALIZED
C FIBONACCI SEQUENCE TYPE GENERATOR WITH LAG 127 REFERRED TO ABOVE AND T
C SUBROUTINE RECUR IN THE ABOVE PAPER WAS CALLED ONCE AGAIN BECAUSE IT W
C OBSERVED THAT THE FIRST 607 OUTPUT NUMBERS WERE STATISTICALLY POOR BUT
C EVERTHING THEREAFTER WAS ACCEPTABLE. SEVERAL OTHER SETS OF INITIAL 19
C ARBITRARY 32-BIT INTEGERS WERE TRIED BUT IN TERMS OF THE STATISTICAL
C PROPERTIES OF THE OUTPUT SEQUENCES THIS SET WAS THE BEST FOUND. THERE
C EVEN BETTER SETS OF INITIAL 19 ARBITRARY 32-BIT INTEGERS (OR 38 ARBITR
C 16-BIT INTEGERS FOR SUBROUTINE INIT16). THIS CRITICAL FACT WAS NOT MEN
C IN THE ABOVE PAPER AND SUGGESTED GOOD SETS OF INITIAL ARBITRARY INTEGE
C WOULD HAVE BEEN APPRECIATED IN THE ABOVE PAPER (INITIAL ATTEMPTS AT
C FINDING THESE WERE DISENCHANTING). THE ACTUAL VALUES IN ARRAY R USED H
C ARE THOSE FOR THE FIRST 607 TERMS OF THE OUTPUT (I.E. AFTER ANOTHER CA
C TO SUBROUTINE RECUR).
C
      INTEGER  J
      INTEGER*4  R(607)

      save r,j ! added 8/11/99

      INTEGER*4 R1(56), R2(56), R3(21), R4(15), R5(55), R6(56), R7(21),
     $          R8(8),  R9(55),R10(56),R11(21),R12(8), R13(55),R14(56),
     $          R15(21),R16(8),R17(39)
      EQUIVALENCE (R1 (1),R(  1)),(R2 (1),R( 57)),(R3 (1),R(113)),
     $            (R4 (1),R(134)),(R5 (1),R(149)),(R6 (1),R(204)),
     $            (R7 (1),R(260)),(R8 (1),R(281)),(R9 (1),R(289)),
     $            (R10(1),R(344)),(R11(1),R(400)),(R12(1),R(421)),
     $            (R13(1),R(429)),(R14(1),R(484)),(R15(1),R(540)),
     $            (R16(1),R(561)),(R17(1),R(569))
       DATA J /0/
C      DATA (R(L), L = 1,56)
       DATA R1/
     $        7946221,6402471,5956499,2794335,4802997,5501139,3040862,
     $        7669582,5450796, 885885,5425723, 841613,4125277,6979174,
     $        3442271,7220713,   2051,6546285,6726828,  98628,4433354,
     $        1773705, 103974,5061898,1573975,3830657,2321578,2696139,
     $        3938712,4564448,4902878,1811534,7634346,8080631,4785742,
     $        1406700,7957107,1951572,5908208,3715704, 439723,6023041,
     $        4571239,5173606,7743011,3468131,5924491,7862638,4625566,
     $        7183511,8206003, 993573,5426025,5024420,6640024,2403484/
C      DATA (R(L), L=57,112) /
       DATA R2/
     $        5886008,7640402,2801935,2390219, 708068, 412442,3838822,
     $        4494121,6267116,5961330,2907175,6132078,5275005, 469647,
     $        2546893,7475449, 944053,8098183,2926846,4649462,4899427,
     $        3327902,4199241,6211274,6794212,6842523, 634529,6215132,
     $        1915772, 945378,1196186,3475780,3042447,2332838,3037376,
     $        1478870,7802292,3958762,7163775,5776535,5277879,3663849,
     $        1108865,3892452, 800884,3214808,6065057,7364409,4503910,
     $        6071818,2630935, 799757,8266804,5412133,4296988,4630575/
C      DATA (R(L), L=113,133)  /
       DATA R3/
     $        3611184,3933490,1968860,7451059,1786737,2897662,2206505,
     $        7712743,2279026,5515309,3782661,6969285, 779351,2926396,
     $         262527,3672812, 443277,5354233,5275354,6678078,8312353/
C      DATA (R(L),L = 134,148) /
       DATA R4/
     $         175705, 933561,7325396, 340306, 842868,5541602,4504054,
     $         4489435,8367356,8167254,3582486,7631028,
     $         690841,1235032,2258194/
C      DATA (R(L),L=149,203) /
       DATA R5/               7935091,8380568,1768489,7002790,1629022,
     $        3796842,5440750,4065994,3866910,5584252, 845925,4510760,
     $        1343629,
     $        6797712,6310891,3921590,2870553,7841205,4591476,7327100,
     $        2695799,1202571,6904100,1947430,4107803,1409777,5336977,
     $        7250137,1094463, 611614,1298499,5623330,6253895,3614627,
     $        5868596,4619568,1963457,3438199,7653407,4178951,4102908,
     $        5021296,7710933,6234390,7363257,5848390, 753806,4438778,
     $        1830569,5611573,1462256,6339979,3906921,5189493,6466286/
C      DATA (R(L),L=204,259) /
       DATA R6/
     $        4691374,5070849,1764306,2293151,4341931,6224483,1900704,
     $        5269125,1684616,3341559,1207528,3873649,6745220, 955493,
     $        2457014,5192591,5515689,8305855,4045748,8060796,5290966,
     $        7432232, 505812,6480772,1995003, 474333,2375699,4190259,
     $        1559579,2427583,7276087,1210713,7316850,7629007,3904425,
     $        1233279,7555913,5327322,7295851, 192443, 763953,3484203,
     $         722297,5245042,5204601,2214343,5141460,3999270,1412356,
     $        5198164,4112697,1276440,2829991,4888598,6541051, 113408/
C      DATA (R(L),L=260,280) /
       DATA R7/
     $        6767965,3796817,8292586,4248682,8195805,7094253,1846382,
     $        5763271, 685901,5992487,7595115,4568268,6455332,2500471,
     $        7660283,2686470,1906124,7821649,2763412,4580711,3543146/
C      DATA (R(L),L=281,288) /
       DATA R8/               2473514, 988025,8115676,  45187,4122015,
     $                                          4680236,34532,5896519/
C      DATA (R(L),L=289,343) /
       DATA R9/               8024493,6125194,4046896, 592873,5465167,
     $                                                        6567655,
     $        1661951,7422893,1079258,3553765,2505499,1276019,2521777,
     $        4380365,4164302,7241077,2888650,2608334,5995258,5195553,
     $        1851528,6542918,5946377,4311821,6304210,7569228,2144343,
     $        1135080,4356899,7353794,6196513,3086768,4459907,5997821,
     $         622281, 944786,3365410,4160816,4943180,3102910,7869259,
     $        2112077,5973734,5050135,  59702,5041693,2389722,3354728,
     $         438099,2341462,3772524,3037847,4875027,5858269, 756865/
C      DATA (R(L),L=344,399)/
       DATA R10/
     $        6579323,2937057,1682304,5436192, 212231,2631277, 493806,
     $        4537370,4562225,  89034,6994185,1911962,3000590,1738673,
     $        5343480,7902738, 607018,7493645,1455799,6891556,6439710,
     $        4566499, 197833,3828187,3565081, 753498,7108413,5909713,
     $         816743,7191655,7705117, 277078,8379964,6522527, 567530,
     $        4333883,3520331,3637242,8293220,6386045,  16635,6404648,
     $         496088,4569307,1424898,6322006, 517135,6683332,3250459,
     $        7343538,6586775,2202723,7287154,2882564,2335689,5434569/
C      DATA (R(L),L=400,420)/
       DATA R11/
     $        7346821,1376968,7329549,5573319,3029050,2817591,1579251,
     $        5461932,3779406,6430667,6742815,8087016,7564320,3537263,
     $        4138750,5015084,2180995,1326894,5280361,4607904,1979819/
C      DATA (R(L),L=421,428) /
       DATA R12/              5117070,8190111,4712301,7790100,7448863,
     $                                        2182144,8300670, 227247/
C      DATA (R(L),L=429,483) /
       DATA R13/              1169253,7900939,2500576,2721460,6017808,
     $                                                        3054565,
     $        1884402,8170513,6155220,  71388,4011744,8087776,2688542,
     $        1072914,2764338,5019945,7740000,2126353,4161465,3307342,
     $        5866788,6579645, 957941,2857357,5955781,5171108,5130601,
     $         777584,5018990,2283082,6435247,3504433,7454958,1170954,
     $        6350704,7349616,5069849,6927822,5442012,2971089,6738859,
     $        1026637, 669882, 426592,3648755, 584387,5585967,4555106,
     $        1332896,2792055,4754270,4812634,1905178,1263748,5195926/
C      DATA (R(L),L=484,539)/
       DATA R14/
     $        3209117,5626804,2953249,6941796,5972126,3017334,1324316,
     $        5230327,2394723,2709887,4024646,1337840,4182440,5793448,
     $          44645,6624734,2426212,4197344,3993943,1612938,4413172,
     $        5900588,4944521,3883535, 312406,6090316,5019855,2007011,
     $        5703550, 782705, 617973,4160477,4628189,5595237,7523986,
     $        2056632,2696995,8164553,3051266,6133032,7203843,2727056,
     $         109774,7171785,4970311,3782449,6232155,7000603,3721999,
     $        4407501,7049719,4505604,3123394,4896828,5597737,3246718/
C      DATA (R(L),L=540,560)/
       DATA R15/
     $        3328082, 274679,4832728,5228997,3409248, 136065, 119828,
     $        1707476,6187228,2638276,1614847,6865684, 882967,3213668,
     $        6900189,3290693,6492823,2648319,3184716,6364279,2936742/
C      DATA (R(L),L=561,568) /
       DATA R16/              7176030, 110744,7545271,6121596,4885521,
     $                                         704872,1712955,6201023/
C      DATA (R(L),L=569,607) /
       DATA R17/              4674124,2227703,3735712,6373528,4352149,
     $                                                        3202547,
     $        5162068,3298204,5787796,5005306,4600830,1855817,1862179,
     $        2700018,4897780,3704900,5944567,1729547, 772971,5630542,
     $        4577102,3965458,7873051,2940593,3518829,3682912,8223045,
     $        2345503,3829485,5994128,1619820,4102434,1841586,3814261,
     $        2004975,7696833,6860599, 399604,4041012 /


C SHOULD WE GENERATE THE NEXT 607 TERMS OF THE SEQUENCE?
      J=J+1
      IF (J.LE.607) GO TO 40
C
C GENERATE THE NEXT 607 TERMS OF THE SEQUENCE. THE SEQUENCE IS BASED ON
C                        607  334
C PRIMITIVE TRINOMIAL   X   +X   +1  OVER GF(2).
C
10    DO 20 I=1,273
20    R(I)=IEOR(R(I),R(I+334))
      DO 30 I=274,607
30    R(I)=IEOR(R(I),R(I-273))
C REINITIALIZE J
      J=1
C SCALE THE RESULT TO [0,1] BY DIVIDING BY 2**23 (AS ARBITRARY 23-BIT IN
C ARE PRODUCED)
40    RANDT=FLOAT(R(J))/8388608.0
      RETURN
      END




      REAL*4  FUNCTION Rnd3(IDUM)
C the seed IDUM should be some large negative integer on the first call
C and the return value should not be altered in the main program thereafter
C         IMPLICIT REAL*4(M)
C         PARAMETER (MBIG=4000000.,MSEED=1618033.,MZ=0.,FAC=2.5E-7)
      integer*4 idum

      PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,FAC=1.E-9)
      DIMENSION MA(55)
      DATA IFF /0/
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
        MJ=MSEED-IABS(IDUM)
        MJ=MOD(MJ,MBIG)
        MA(55)=MJ
        MK=1
        DO 11 I=1,54
          II=MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK
          IF(MK.LT.MZ)MK=MK+MBIG
          MJ=MA(II)
11      CONTINUE
        DO 13 K=1,4
          DO 12 I=1,55
            MA(I)=MA(I)-MA(1+MOD(I+30,55))
            IF(MA(I).LT.MZ)MA(I)=MA(I)+MBIG
12        CONTINUE
13      CONTINUE
        INEXT=0
        INEXTP=31
        IDUM=1
      ENDIF
      INEXT=INEXT+1
      IF(INEXT.EQ.56)INEXT=1
      INEXTP=INEXTP+1
      IF(INEXTP.EQ.56)INEXTP=1
      MJ=MA(INEXT)-MA(INEXTP)
      IF(MJ.LT.MZ)MJ=MJ+MBIG
      MA(INEXT)=MJ
      Rnd3=MJ*FAC
      RETURN
      END

      
      subroutine dotime(i1,i2,i3)  ! comment this out if  itime is available
c     subroutine test_itime !

c     Sun Sep 27 17:04:09 IST 1998
c     12345678901234567890
      integer*4     i1,i2,i3    !output

      character*20  time        !internal
      character*1   ch1
      byte          j(3)
      byte          k           ! internal
      equivalence   (ch1,k)

      i0= 12 ! first character

      call system('unset noclobber; date > .tmp; set noclobber')
      open(1,file='.tmp',access='sequential',
     _     status='old')
      read(1,'(a)')time
      close(1)

      do jj=3,20
         ch1=time(jj:jj)
         j((jj-3)/6+1)=j((jj-3)/6+1)+int(k)
      enddo
      i1= 300+ j(1)
      i2= 300+ j(2)
      i3= 300+ j(3)
      return
      end

